
clear all
close all
clc

% ADD ALL FUNCTION PATHS
addpaths

%% INPUT PARAMETERS FOR THE TESTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SOURCE_FOLDER = 'images/source'; % folder where the target images are located
folder = './globalResults/';
OUTPUT_FOLDER = 'images/hippocampi';
REF_FOLDER = 'images/reference';
REF_CONT_FOLDER = 'images/reference_cont';
ELAB_FOLDER = 'images/elab';
ELAB_CONT_FOLDER = 'images/elab_cont';
FILENAME_TIME = 'images/execution_times.xlsx';
fileResultsLocalizationDG = 'localizationResultsDG';
fileResultsLocalizationCA = 'localizationResultsCA';
MODEL_TO_COMPARE = 'consensus'; % 'intersection','union','consensus','STAPLE'

global databaseOfMethods;
databaseOfMethods = {'IO','LS','IOECL','FCM','ST','Otsu','AS','CV','GAC','DS', 'KM','GACseeds','CVseeds','DSGAC','DSCV','newHybrid'}; %list of segmentation methods we can test
% 'IO' -> Iterative Otsu + Active Shape Models (NON-DETERMINISTIC)
% 'LS' -> Level Set Method (half-Chan&Vese like) + Active Shape Models (NON-DETERMINISTIC)
% 'IOECL' -> Iterative Otsu + Active Shape Models + Random Forests (NON-DETERMINISTIC)
% 'FCM' -> Fuzzy C-Means (DETERMINISTIC)
% 'ST' -> Soft Thresholding (DETERMINISTIC)
% 'Otsu' -> Otsu Thresholding Method (DETERMINISTIC)
% 'AS' -> Affine Segmentation (affine registration using Scatter Search) (NON-DETERMINISTIC)
% 'DRLSE' -> Distance Regularized Level Set Evolution (DETERMINISTIC)
% 'CV' -> Chan & Vese Level Set model (DETERMINISTIC)
% 'GAC' -> Geodesic Active Contour (DETERMINISTIC)
% 'GACseeds' --> GAC from ASM seeds (NON-DETERMINISTIC)
% 'CVseeds' --> CV from ASM seeds (NON-DETERMINISTIC)
% 'DS' -> Deformable Segmentation (affine registration using Scatter Search + B-splines using Gradient Descent) (NON-DETERMINISTIC)
% 'KM' -> K-Means (DETERMINISTIC)
% 'DSGAC' -> Deformable Segmentation + Geodesic Active Contours (NON-DETERMINISTIC)
% 'DSCV' -> Deformable Segmentation + Chan&Vese (NON-DETERMINISTIC)
% 'newHybrid' -> LS (edge-based (VFC) + region-based (local Chan&Vese) + prior-shape (DS)) (NON-DETERMINISTIC)
global metricsUsed;
metricsUsed = {'TP','FP','FN','DSC','HD','J','segmentationTime','totalTime'};

previousRead = 1; % If previousRead is 1 you will read the image from here, not in getHippocampus every run
performSegmentation=1; % if performSegmentation = 0 only the localization will be done
% --> interesting in order to compare different metaheuristics during localization
% -------------------------------------------------------------------------
segmentationMethods = {'CV'};% 'IO','LS','IOECL','FCM','ST', 'Otsu', 'AS','CV','GAC','DS', 'KM','GACseeds', 'CVseeds', 'DSGAC','DSCV','newHybrid'
NUMBER_OF_TESTS = 1; % number of runs/segmentations per image
All_ImageModality = {'ISH'}; % 'ISH', 'Lungs', 'Knee', 'MRI' %Lungs and Knee are CT
% -------------------------------------------------------------------------
ValidateSegmentationResults = 1; % VALIDATE SEGMENTATION RESULTS??
SaveSegmentationEvolution = 0; % SAVE EVOLUTION OF THE SEGMENTATIONs???

modelsToUse = {'ASC'};%,'ASC','morePoints'};
% 'classical' -> old models (8 CA + 7 SG)
% 'ASC' -> longer models for the images in the Applied Soft Computing paper
% 'morePoints' -> variable length of the models (because we saw that usually CA is not completely covered)
%       The number of points in CA are: 10, 10, 9, 9, 9, 9, 9, 9, 8, 8
%       (depending on the slice: 9 to 18)
%       While the number of points in DG is always 7
numberModelsToTest = 5; % only useful if referencesToUse ~= 0;
% if referencesToUse==0 -> the maximum number of models to test will be 5

metaheuristic = {'DE'}; %'DE' 'GA' 'PSO' 'SS' 'SA'
generations = 350;
populationSize = 150;
% number of repetitions of the stochastic optimization algorithm: this is
% for comparing the different metaheuristics.
% If you want to compare only the segmentation
% result put this variable to 1.
metaheuristicRuns = 1;
% if we have more than one metaheuristic (or more than 1 metaheuristic repetition), that means we want to compare
% metaheuristics during localization, not segment the image (since we will
% do the segmentation only with the last metaheuristic in the list), so
if (size(metaheuristic, 2) > 1) | (metaheuristicRuns > 1)
    performSegmentation=0;
end;
if performSegmentation == 0
    ValidateSegmentationResults = 0;
end;

registration_type = 'globalAndLocal'; % 'global', 'globalAndLocal'
% we're going to apply our localization method over a ROI, but what kind of
% ROI?
% - 'raw': original ROI resized and grey-levels
% - 'preprocessing1': combination of classical computer vision techniques
% defined in preprocess_roi.m
typeOfROI = 'preprocessing1moreNoisy'; % 'raw', 'preprocessing1', 'preprocessing1moreNoisy', 'likeLocalRegistration'
extract_contours = 0; %if 0 we don't do the contour extraction (we suppose it has been already done)
show_output = 0;
writeTimeExcel = 2; %0 no time output in Excel file, 1 output creating new file, 2 output appending the existent file
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for indexModality = 1:size(All_ImageModality,2)
    ImageModality = char(All_ImageModality(indexModality));
    
    %% DATASETS
    if strcmp(ImageModality,'ISH')
        %% real images
        % image_name = {'1300018I05Rik_86','0610010D24Rik_103_26','1300018I05Rik_94','5430437P03Rik_107_26','reference12','Atrx_133_32','B230215L15Rik_129','reference15','Nmt1_140_34','reference13','Wbp7_110','Wars_109','Tubb3_114','Trem1_2800','reference14','3830406C13Rik_66','A030009H04Rik_117','Atp1b2_2725','Azin1_96','Camk2a_102','Camk2b_102','Camk2g_128_2248','Cd9_30','Cutl2_30','Gad1_104','Gapdh_93','Gfap_95_2307','Mbp_130','Slc17a7_120','Zim1_117'};
        % % image_name = {'0610010D24Rik_103_26', '5430437P03Rik_107_26', 'reference12', 'Atrx_133_32', 'Nmt1_140_34', 'reference13', 'Trem1_2800'};
        % % referencesToUse = [10, 9, 12, 12, 14, 13, 15];
        % referencesToUse = [14,10, 15,9,12,12,16,15,14,13,15,16,16,15,14,16, 14, 17, 17, 15, 12, 17, 10, 11, 14, 12, 16, 17, 12, 16];
        % referencesToUse = 0;
        
        %% images to use in the comparison with andrea
%         image_name = {'Wbp7_110','1300018I05Rik_86','0610010D24Rik_103_26','1300018I05Rik_94','5430437P03Rik_107_26','B230215L15Rik_129','Nmt1_140_34','Wars_109','Tubb3_114','Trem1_2800','3830406C13Rik_66','A030009H04Rik_117','Atp1b2_2725','Azin1_96','Camk2a_102','Camk2b_102','Camk2g_128_2248','Cd9_30','Gad1_104','Gapdh_93','Mbp_130','Slc17a7_120'};
%         referencesToUse = [15,14,10, 15,9,16,14,16,16,15,16, 14, 17, 17, 15, 12,17, 10, 14, 12, 17, 12];
   
        
                        % tests Davide Valeriani e Alessandro Costalunga: MedGEC paper
                        image_name = {'reference11','1300018I05Rik_86','Gad1_104','reference14','reference15','reference16','reference17','Tubb3_114','Cd9_30','reference12'};
                        referencesToUse = [11,14,14,14,15,16,17,16,10,12];
        
        %     image_name = {'Wbp7_110','1300018I05Rik_86','0610010D24Rik_103_26','1300018I05Rik_94','5430437P03Rik_107_26','Atrx_133_32','B230215L15Rik_129','Nmt1_140_34','Wars_109','Tubb3_114','Trem1_2800','3830406C13Rik_66','A030009H04Rik_117','Atp1b2_2725','Azin1_96','Camk2a_102','Camk2b_102','Camk2g_128_2248','Cd9_30','Cutl2_30','Gad1_104','Gapdh_93','Gfap_95_2307','Mbp_130','Slc17a7_120','Zim1_117'};
        %     referencesToUse = [15,14,10, 15,9,12,16,14,16,16,15,16, 14, 17, 17, 15, 12, 17, 10, 11, 14, 12, 16, 17, 12, 16];
        
        % Bad images for GAC
        % image_name = {'Trem1_2800','3830406C13Rik_66','Cd9_30','Cutl2_30','Gfap_95_2307','Mbp_130','Slc17a7_120','Zim1_117', 'B230215L15Rik_129'};
        % referencesToUse = [15,16,10, 11, 16, 17, 12, 16,16];
        
        % Bad images for CV
        %     image_name = {'Cutl2_30','Gfap_95_2307','Mbp_130','Zim1_117','Atrx_133_32','Cd9_30'};
        %     referencesToUse = [11, 16, 17, 16, 12,10];
        % image_name = {'Mbp_130','Cutl2_30', 'Gfap_95_2307', 'Zim1_117','Atp1b2_2725','Wbp7_110','Atrx_133_32'};
        % referencesToUse = [17,11, 16, 16,17,15,12];
        
        % Bad images for ST
        %     image_name = {'1300018I05Rik_86','Wars_109', 'Camk2g_128_2248', 'Atp1b2_2725','Camk2a_102','Mbp_130','Cutl2_30', 'Gfap_95_2307', 'Zim1_117','Wbp7_110','Atrx_133_32'};
        %     referencesToUse = [14,16,17,17,15,17,11,16,16,15,12];
        %     image_name = {'Gfap_95_2307', 'Zim1_117','Atrx_133_32'};
        %     referencesToUse = [16,16,12];
        
        % worst images
        % image_name = {'0610010D24Rik_103_26','B230215L15Rik_129','Nmt1_140_34','Wars_109','Atp1b2_2725','Cd9_30','Gfap_95_2307','Mbp_130','Zim1_117'};
        % referencesToUse = [10,16,14,16,17,10, 16, 17,16];
        % image_name = {'0610010D24Rik_103_26','Nmt1_140_34'};
        % referencesToUse = [10,14];
        
        % image_name = {'reference10','reference11','reference16','reference17'};
        % referencesToUse = [10,11,16,17];
        
        %% synthetic images
        % image_name =  {'Atlas-Sagittal-26-A.jpg_synthetic1','Atlas-Sagittal-26-A.jpg_synthetic2', 'Atlas-Sagittal-26-A.jpg_synthetic3' ...
        %     'Atlas-Sagittal-28-A.jpg_synthetic1','Atlas-Sagittal-28-A.jpg_synthetic2','Atlas-Sagittal-28-A.jpg_synthetic3', ...
        %     'Atlas-Sagittal-30-A.jpg_synthetic1','Atlas-Sagittal-30-A.jpg_synthetic3','Atlas-Sagittal-30-A.jpg_synthetic3', ...
        %     'Atlas-Sagittal-32-B.jpg_synthetic1','Atlas-Sagittal-32-B.jpg_synthetic2','Atlas-Sagittal-32-B.jpg_synthetic3', ...
        %     'Atlas-Sagittal-34-B.jpg_synthetic1','Atlas-Sagittal-34-B.jpg_synthetic2','Atlas-Sagittal-34-B.jpg_synthetic3', ...
        %     'Atlas-Sagittal-36-A.jpg_synthetic1','Atlas-Sagittal-36-A.jpg_synthetic2','Atlas-Sagittal-36-A.jpg_synthetic3','Atlas-Sagittal-36-A.jpg_synthetic4', ...
        %     'Atlas-Sagittal-38-C.jpg_synthetic1','Atlas-Sagittal-38-C.jpg_synthetic2','Atlas-Sagittal-38-C.jpg_synthetic3', ...
        %     'Atlas-Sagittal-40-D.jpg_synthetic1','Atlas-Sagittal-40-D.jpg_synthetic2','Atlas-Sagittal-40-D.jpg_synthetic3','Atlas-Sagittal-40-D.jpg_synthetic4', ...
        %     'Atlas-Sagittal-42-D.jpg_synthetic1', 'Atlas-Sagittal-42-D.jpg_synthetic2', ...
        %     'Atlas-Sagittal-44-D.jpg_synthetic1','Atlas-Sagittal-44-D.jpg_synthetic2'};
        % referencesToUse = [9, 9, 9, 10, 10, 10, 11, 11, 11, 12, 12, 12, 13, 13, 13, 14, 14, 14, 14, 15, 15, 15, 16, 16, 16, 16, 17, 17, 18, 18];
        
    elseif strcmp(ImageModality,'MRI')
        %     image_name = {'sujet01_2_50', 'sujet01_2_55','sujet01_2_60','sujet02_2_50','sujet02_2_55','sujet02_2_60','sujet03_2_60','sujet03_2_65','sujet03_2_70','sujet04_2_60','sujet04_2_65','sujet04_2_70','sujet05_2_55','sujet05_2_60','sujet05_2_65','sujet06_2_65','sujet06_2_70'};
        image_name = {'sujet01_2_50', 'sujet01_2_55','sujet01_2_60','sujet02_2_50','sujet02_2_60','sujet03_2_65','sujet03_2_70','sujet04_2_60','sujet04_2_65','sujet05_2_55','sujet05_2_60','sujet05_2_65','sujet06_2_65','sujet06_2_70'};
        referencesToUse = [-1, -1,-1,-1, -1,-1,-1, -1,-1,-1, -1,-1,-1, -1];
   
        %     image_name      = { 'sujet01_2_50', 'sujet03_2_70', 'sujet02_2_50'};
        %     referencesToUse = [-1, -1, -1];
        % These references are -1 because, if not, I had to change more things
        % in getHippocampus. The references for DS and AS are inside such
        % functions. I know this shouldn't be in this way...
        previousRead = 1;
        best_ref=-1;
    elseif strcmp(ImageModality,'Lungs')
        %     image_name = {'l7', 'l6', 'l1', 'l8', 'lungs1', 'lungs2', 'rodilla1', 'rodilla2', 'rodilla3', 'rodilla4'};
        %     referencesToUse = [-1,-1,-1,-1,-1,-1,-1,-1,-1,-1];
        % TEST IMAGES
        image_name      = { 'l6', 'l7', 'l8','lungs1','lungs2'};
        referencesToUse = [ -1,-1,-1,-1,-1];
        image_name      = { 'l8','lungs2'};
        referencesToUse = [-1,-1];        
        previousRead = 1;
        best_ref=-1;
    elseif strcmp(ImageModality,'Knee')
        %     image_name = {'l7', 'l6', 'l1', 'l8', 'lungs1', 'lungs2', 'rodilla1', 'rodilla2', 'rodilla3', 'rodilla4'};
        %     referencesToUse = [-1,-1,-1,-1,-1,-1,-1,-1,-1,-1];
        % TEST IMAGES
        image_name      = { 'rodilla1', 'rodilla2', 'rodilla4'  };
        referencesToUse = [ -1,-1,-1];
        previousRead = 1;
        best_ref=-1;
    end;
    
    NUMBER_OF_IMAGES = size(image_name,2);
    
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% DEPENDENCIES BETWEEN METHODS
    % Default value for refinement
    refinement = 0;
    
    % If we have included 'IOECL' but not 'IO' we have to run also 'IO'
    if (sum(ismember(segmentationMethods,'IOECL'))>0 & (sum(ismember(segmentationMethods,'IO'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'IO';
        refinement = 1; % APPLY REFINEMENT USING ENSEMBLE CLASSIFIERS?
    end;
    
    % If we have included 'GACseeds' but not 'IO' we have to run also 'IO'
    if (sum(ismember(segmentationMethods,'GACseeds'))>0 & (sum(ismember(segmentationMethods,'IO'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'IO'; % We need maskpoly as seed for GAC
        refinement = 0;
    end;
    
    % If we have included 'CVseeds' but not 'IO' we have to run also 'IO'
    if (sum(ismember(segmentationMethods,'CVseeds'))>0 & (sum(ismember(segmentationMethods,'IO'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'IO'; % We need maskpoly as seed for CV
        refinement = 0;
    end;
    
    % If we have included 'DSGAC' but not 'DS' we have to run also 'DS'
    if (sum(ismember(segmentationMethods,'DSGAC'))>0 & (sum(ismember(segmentationMethods,'DS'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'DS';
    end;
    
    % If we have included 'newDS' but not 'DS' we have to run also 'DS'
    if (sum(ismember(segmentationMethods,'newHybrid'))>0 & (sum(ismember(segmentationMethods,'DS'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'DS';
    end;
    
    % If we have included 'DSCV' but not 'DS' we have to run also 'DS'
    if (sum(ismember(segmentationMethods,'DSCV'))>0 & (sum(ismember(segmentationMethods,'DS'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'DS';
    end;
    
    % If we have included 'DS' but not 'AS' we have to run also 'AS'
    if (sum(ismember(segmentationMethods,'DS'))>0 & (sum(ismember(segmentationMethods,'AS'))==0))
        segmentationMethods{size(segmentationMethods,2)+1} = 'AS';
    end;
    
    %%
    
    for numberModelsToUse = 1:size(modelsToUse,2)
        
        bestReferenceSliceTemporal = {};
        evaluatedRegionsIOECLTemporal = {};
        
        allResultsLocalizationCA = {};
        allResultsLocalizationDG = {};
        
        localizationTimeTemporal = {};
        allResultsSegmentation = {};
        
        for i=1:NUMBER_OF_IMAGES % Total number of images
            %% READING IMAGES
            if previousRead % We will read the image from here
                if isequal(referencesToUse,0) % We are going to use registration to extract the ROI
                    source_image_name = sprintf('%s/%s.jpg',SOURCE_FOLDER,char(image_name(i)));
                    try
                        Iorig = cvReadImage(source_image_name,0);%rgb2gray(imread(source_image_name));
                    catch
                        Iorig = imread(source_image_name);
                        if (size(Iorig,3)==3)
                            Iorig = rgb2gray(imread(source_image_name));
                        end;
                        disp('cvReadImage not available!');
                    end;
                else % We are not going to apply registration to extract the ROI: we pass the ROI image directly
                    hip_image_name = sprintf('%s/%s_ROI.png',SOURCE_FOLDER,char(image_name(i)));
                    try
                        Iorig = cvReadImage(hip_image_name,0);
                    catch
                        Iorig = imread(hip_image_name);
                        if (size(Iorig,3)==3)
                            Iorig = rgb2gray(imread(source_image_name));
                        end;
                        disp('cvReadImage not available!');
                    end;
                end;
            else % We will read the image directly from getHippocampus
                Iorig=0;
            end;
            
            for j=1: NUMBER_OF_TESTS
                %% RUNNING OF THE ALGORITHM
                if isequal(referencesToUse,0)
                    [finalModelDG finalModelCA individualResults] = ...
                        getHippoCampus(char(image_name(i)), extract_contours, show_output, refinement,writeTimeExcel, ...
                        segmentationMethods,char(modelsToUse(numberModelsToUse)),Iorig,0, ...
                        metaheuristic, registration_type, SaveSegmentationEvolution, ValidateSegmentationResults, ...
                        typeOfROI, metaheuristicRuns, performSegmentation, SOURCE_FOLDER, OUTPUT_FOLDER, ...
                        REF_FOLDER, REF_CONT_FOLDER, ELAB_FOLDER, ELAB_CONT_FOLDER, FILENAME_TIME, numberModelsToTest, ...
                        generations, populationSize,MODEL_TO_COMPARE,ImageModality);
                else
                    [finalModelDG finalModelCA individualResults] = ...
                        getHippoCampus(char(image_name(i)), extract_contours, show_output, refinement,writeTimeExcel, ...
                        segmentationMethods,char(modelsToUse(numberModelsToUse)),Iorig,referencesToUse(i), ...
                        metaheuristic, registration_type, SaveSegmentationEvolution, ValidateSegmentationResults, ...
                        typeOfROI, metaheuristicRuns, performSegmentation, SOURCE_FOLDER, OUTPUT_FOLDER, ...
                        REF_FOLDER, REF_CONT_FOLDER, ELAB_FOLDER, ELAB_CONT_FOLDER, FILENAME_TIME, numberModelsToTest, ...
                        generations, populationSize,MODEL_TO_COMPARE,ImageModality);
                end;
                
                % individualResults make reference to the results obtained with
                % one image. Single metrics (not shared for all segmentation
                % algorithms or metaheuristics):
                % bestReferenceSlice, localizationTime, evaluatedRegions
                bestReferenceSliceTemporal = [bestReferenceSliceTemporal, {individualResults.bestReferenceSlice}];
                allResultsSegmentation = [allResultsSegmentation {individualResults}];
                allResultsLocalizationCA = [allResultsLocalizationCA {individualResults.localizationCA}];
                allResultsLocalizationDG = [allResultsLocalizationDG {individualResults.localizationDG}];
                
                if ((j==1) && (writeTimeExcel==1))
                    writeTimeExcel = 2;
                end;
            end;
        end;
        
        %% STATISTICS CALCULATION %%
        [globalResults] ...
            = calculatingGlobalResults(folder, NUMBER_OF_IMAGES, NUMBER_OF_TESTS, metaheuristic, segmentationMethods, ...
            allResultsLocalizationCA, allResultsLocalizationDG, allResultsSegmentation, fileResultsLocalizationDG, ...
            fileResultsLocalizationCA,performSegmentation,ImageModality);
        
        if performSegmentation == 1
            %% SHOWING RESULTS
            displaySegmentationResults(globalResults, segmentationMethods);
        end;
        
        %% PERFORMING STATISTICAL TESTS
        performStatisticalTests(allResultsSegmentation, segmentationMethods, fileResultsLocalizationDG, fileResultsLocalizationCA, ...
            folder, metaheuristic, NUMBER_OF_IMAGES, NUMBER_OF_TESTS,performSegmentation);
        
        fclose('all');
    end
    
end;


