
function [fxmin, xmin, history] = ScatterSearch2(fun, options)
global FEVAL_CALLS;
%% SCATTER SEARCH PARAMETERS
n_elems                    = options.n_elems;
lb                         = options.lb;
ub                         = options.ub;
refset_best_size           = options.refset_best_size;
refset_diverse_size        = options.refset_diverse_size;
population_size            = options.population_size;%(refset_best_size + refset_diverse_size)*10;
% initial_improvement_method = 0;
max_num_reconstructions    = options.max_num_reconstruction;
alpha                      = options.alpha_blx;  %for BLX-alpha
n_parents                  = options.n_parents;
max_evals                  = options.max_evals;
max_iter                   = options.max_iters;
selection_method           = options.selection_method;
local_search_method        = options.local_search_method;
history                    = zeros(options.max_evals/options.save,1);

%% SCATTER SEARCH GENERATION OF SOLUTIONS
% Generate an initial set of solutions to guarantee a
% critical level of diversity. In this case: Uniformly distributed
% pseudorandom numbers
initial_solutions = rand(population_size,n_elems).*(ub-lb)+lb;
Population = initial_solutions;
FEVAL_CALLS = 0;
historyInd = 1;

%% REFERENCE SET - INITIALIZATION
% Designate a subset of best solutions as the
% reference set (best in terms of objective function
% value or diversification)
% reference_set = [];
% initial_fitness = [];
% reference_set_fitness = [];
% for i=1:population_size
%     x0 = Population(i,:);
%     % Evaluate the fitness function
%     initial_fitness = [initial_fitness; feval(fun, x0)];
%     FEVAL_CALLS = FEVAL_CALLS + 1;
% end
initial_fitness = feval(fun, Population);
FEVAL_CALLS = FEVAL_CALLS + length(Population);
history(historyInd) = min(initial_fitness);
historyInd = historyInd + 1;
% Best solutions from P (QUALITY)
[temp,idx] = sort(initial_fitness);
reference_set=Population(idx(1:refset_best_size),:);
reference_set_fitness = initial_fitness(idx(1:refset_best_size),1);
Population(idx(1:refset_best_size),:) = [];
% initial_fitness(idx(1:refset_best_size),:) = [];

% Most diverse solutions from P (DIVERSIFICATION): we apply the median to
% the euclidean distances
% dist_vector = zeros(refset_diverse_size,1);
diversity=mean(dist(Population,reference_set(1:refset_best_size,:)'),2);
[temp2,idx2] = sort(diversity, 'descend');
reference_set(refset_best_size+1:refset_best_size+refset_diverse_size,:) = Population(idx2(1:refset_diverse_size),:);
% dist_vector = temp2(1:refset_diverse_size);

%% REFERENCE SET SOLUTION IMPROVEMENT
% optionsi.StopTemp=1e-3;
% optionsi.Verbosity=1;
% optionsi.MaxEvals = 600;
% for h=1:size(reference_set,1)
%     x0 = reference_set(h,:);
%     % Evaluate the fitness function
%     [minimum_rf,fval_rf] = anneal(fun, x0,optionsi);
%     reference_set(h,:) = minimum_rf;
%     reference_set_fitness(h,:) = fval_rf;
% end

%create distance vector
% for ind=1:refset_diverse_size
%     dist_vector(ind) = mean(dist(reference_set(refset_best_size+ind,:), reference_set'));
% end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                      SCATTER SEARCH MAIN LOOP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
termination = 0;
iterations = 0;
number_of_reconstructions = 0;
fprintf(1,'Best Fitness (0): %f\n',min(reference_set_fitness(1:refset_best_size)));

%% GET ALL POSSIBLE COMBINATIONS OF ELEMENTS IN REFERENT SET
all_possible = combntns(1:size(reference_set,1),n_parents);
% old_elements_changed = ones(1,refset_best_size + refset_diverse_size);
elements_changed = ones(size(reference_set,1));
best_fvalue = 1000;
while (termination==0)
%     sum(old_elements_changed(1:refset_best_size))
%     changes = 0;
%     all_possible = combntns(elements_changed(elements_changed>0),n_parents);
%     selected = randperm(size(all_possible,1));
%     new_elements_changed = zeros(1,refset_best_size + refset_diverse_size);
    %% SOLUTION COMBINATION METHOD
    all_solutions = zeros(size(all_possible,1),n_elems,n_parents);
    for i=1:n_parents
        all_solutions(:,:,i) = reference_set(all_possible(:,i),:);
    end
    old_set = reference_set;
        cmax = max(all_solutions,[],3);
        cmin = min(all_solutions,[],3);
    if selection_method == 1
        new_solution = rand(size(all_possible,1),n_elems).*(cmax - cmin + 2*alpha*(cmax-cmin))+(cmin - alpha*(cmax-cmin));
    else
        new_solution = mean(all_solutions,3);
    end
    new_solution(sum(cmax - cmin,2) < 0.001,:) = repmat(reference_set(refset_best_size+refset_diverse_size,:),sum(sum(cmax - cmin,2) < 0.001),1);
    new_solution = new_solution(sum(elements_changed(all_possible),2)>0,:); %remove the combs in which both are not changed in the last iterations
    new_solution(new_solution<lb) = lb;new_solution(new_solution>ub) = ub;
    if local_search_method == 1
        [new_solution fvalue]=my_descent(fun,new_solution);
%         lsqnonlin(fun,new_solution,[],[],optimset('Display','iter','TolX',0,'MaxIter',30000,'TolFun',0,'MaxFunEvals', 20, 'Algorithm','levenberg-marquardt'));
        FEVAL_CALLS = FEVAL_CALLS + 2*size(new_solution,1);
    elseif local_search_method == 2
        fvalue = zeros(size(new_solution,1),1);
        for sol=1:size(new_solution,1)
            [new_solution(sol,:) fvalue(sol) exitflag outp]=fminsearch(fun,new_solution(sol,:),optimset('Display','off','TolX',0,'MaxFunEvals',n_elems+2));
%            [new_solution(sol,:), fvalue(sol), exitflag, outp ]=patternsearch(@(x)pso_find(x),new_solution(sol,:),[],[],[],[],zeros(1,n_elems),ones(1,n_elems),[],psoptimset('Display','off','MaxFunEvals',2,'InitialMeshSize',0.5));
            FEVAL_CALLS = FEVAL_CALLS + outp.funcCount;
            if(FEVAL_CALLS > options.save * historyInd)
                history(historyInd) = min([min(fvalue); best_fvalue]);
                historyInd = historyInd + 1;
            end
        end
    elseif local_search_method == 3
        fvalue = zeros(size(new_solution,1),1);
        for sol=1:size(new_solution,1)
            [new_solution(sol,:) foo fvalue(sol) exitflag outp]=lsqnonlin(fun,new_solution(sol,:),[],[],optimset('Display','off','TolX',0,'MaxIter',30000,'TolFun',0,'MaxFunEvals', n_elems+2, 'Algorithm','levenberg-marquardt'));
%             [new_solution(sol,:), fvalue(sol), exitflag, outp ]=patternsearch(@(x)pso_find(x),new_solution(sol,:),[],[],[],[],zeros(1,n_elems),ones(1,n_elems),[],psoptimset('Display','off','MaxFunEvals',2,'InitialMeshSize',0.5));
            FEVAL_CALLS = FEVAL_CALLS + outp.funcCount;
        end
    else
        fvalue = feval(fun, new_solution);
        FEVAL_CALLS = FEVAL_CALLS + size(new_solution,1);
    end
    best_trial = [reference_set(1:refset_best_size,:);new_solution];
    best_fvalue = [reference_set_fitness;fvalue];
    [tmp idx] = sort(best_fvalue);
    reference_set(1:refset_best_size,:) = best_trial(idx(1:refset_best_size),:);
    reference_set_fitness = tmp(1:refset_best_size);
    %update distance vector
    dist_trial = [reference_set(1+refset_best_size:refset_best_size+refset_diverse_size,:);new_solution];
    dist_fvalue = mean(dist(dist_trial, reference_set(1:refset_best_size,:)'),2);
%     dist_fvalue(1:refset_diverse_size) = dist_vector;
    [tmp idx] = sort(dist_fvalue,'descend');
    reference_set(1+refset_best_size:refset_best_size+refset_diverse_size,:) = dist_trial(idx(1:refset_diverse_size),:);
%     dist_vector = tmp(1:refset_diverse_size);
%     reference_set(1+refset_best_size:refset_best_size+refset_diverse_size,:)
%     reference_set(1:refset_best_size,:)
    changes = any(sum(reference_set ~= old_set,2));
    elements_changed = sum(reference_set ~= old_set,2) > 0;
%     app = sum(cmax - cmin,2) < 0.001;
%     sum(app(1:77))
%     sum(app(78:105))
%     sum(elements_changed)
    
%     for i=1:size(all_possible,1)
%         % In this case, we generate new solutions by means of a combination
%         % of the old solutions (calculating the mean of the values)
%         if sum(old_elements_changed(all_possible(i,:)))==0
%             continue
%         end
% %         new_solution = mean([reference_set(all_possible(selected(i),1),:);reference_set(all_possible(selected(i),2),:)]);
%         cmax = max(reference_set(all_possible(i,1:n_parents),:)); %blx-alpha
%         cmin = min(reference_set(all_possible(i,1:n_parents),:));
%         new_solution = rand(1,n_elems).*(cmax - cmin + 2*alpha*(cmax-cmin))+(cmin - alpha*(cmax-cmin));
%         new_solution(new_solution<lb) = lb;new_solution(new_solution>ub) = ub;
%         if (loop_improvement_method==1)
% %                 options2.StopTemp=1e-2;
% %                 options2.Verbosity=0;
% %                 options2.MaxEvals = 100;
% %                 [new_solution,fvalue] = anneal(fun, new_solution,options2);
%             [new_solution,fvalue] = my_descent(fun, new_solution);
%             FEVAL_CALLS = FEVAL_CALLS + 2;
% %             [new_solution fvalue exitflag optout]=fminsearch(fun,new_solution,optimset('Display','off','TolX',0,'MaxFunEvals',30));
% %             FEVAL_CALLS = FEVAL_CALLS + optout.funcCount;
%         else
%             fvalue = feval(fun, new_solution);
%             FEVAL_CALLS = FEVAL_CALLS + 1;
%         end;
%         %see if it is one of the best
%         [max_value,index] = max(reference_set_fitness(1:refset_best_size));
%         if fvalue < max_value
%             reference_set(index,:) = new_solution;
%             reference_set_fitness(index,:) = fvalue;
%             if new_elements_changed(index) == 0
%                 changes = changes + 1;
%                 new_elements_changed(index) = 1;
%             end
%             
%         end;
%         fdist = mean(dist(new_solution, reference_set(1:refset_best_size,:)'));
%         %see if it is one of the most diverse
%         [min_dist,index] = min(dist_vector);
%         if (fdist > min_dist)
%             dist_vector(index) = fdist;
%             reference_set(index+refset_best_size,:) = new_solution;
%             reference_set_fitness(index+refset_best_size,:) = fvalue;
%             if new_elements_changed(index + refset_best_size) == 0
%                 changes = changes + 1;
%                 new_elements_changed(index + refset_best_size) = 1;
%             end
%         end;        
%     end;
    
    
%     old_elements_changed = new_elements_changed;
    if (changes == 0)
        fprintf(1,'Reconstruction!\n');
        number_of_reconstructions = number_of_reconstructions + 1;
    end
    if ((changes == 0)&&(number_of_reconstructions >= max_num_reconstructions))
        termination = 1;
    end;
    iterations = iterations + 1;
    if ((iterations > max_iter)||(number_of_reconstructions >= max_num_reconstructions) || FEVAL_CALLS >= max_evals)
        termination = 1;
    end;
    if mod(iterations,1)==0
        fprintf(1,'Best Fitness (%d): %f - %d evaluations\n',iterations, min(reference_set_fitness(1:refset_best_size)), FEVAL_CALLS);
    end
%     fprintf(1,'Best Fitness: %f\nVariance of BestSet: %f\nVariance of DiverseSet: %f\n',min(reference_set_fitness(1:refset_best_size)),mean(std(reference_set(1:refset_best_size,:))),mean(std(reference_set(1+refset_best_size:refset_best_size+refset_diverse_size,:))));
    %% REFERENCE SET RECONSTRUCTION
    if ((changes == 0)&&(number_of_reconstructions < max_num_reconstructions)&&(termination==0)) || ((iterations > max_iter)&&(number_of_reconstructions < max_num_reconstructions)&&(termination==0))
        Population = rand(population_size,n_elems).*(ub-lb)+lb;
        diversity=mean(dist(Population,reference_set(1:refset_best_size,:)'),2);
        [temp2,idx2] = sort(diversity, 'descend');
        reference_set(refset_best_size+1:refset_best_size+refset_diverse_size,:) = Population(idx2(1:refset_diverse_size),:);
%         dist_vector = temp2(1:refset_diverse_size);
        elements_changed = ones(size(reference_set,1));
%         old_elements_changed = ones(size(reference_set,1));
    end
end

fprintf(1,'Number of iterations: %d\nNumber of reconstruction: %d\nNumber of Evaluations: %d\n',iterations,number_of_reconstructions, FEVAL_CALLS);

%% RESULTS
[fxmin,final_index] = min(reference_set_fitness);
% xmin_value = reference_set(final_index,:);
% optionsf.Verbosity=1;
% optionsf.StopTemp = 5e-8;
% optionsf.MaxEvals = 10000;
% [xmin,fxmin] = anneal(fun, xmin_value,optionsf);
xmin = reference_set(final_index,:);

