clear all
close all
% clc

%% Tuning made on:
% -- ISH:
% image_name = {'Atrx_133_32', 'Cutl2_30', 'Gfap_95_2307', 'Zim1_117'};
% referencesToUse = [12, 11, 16, 16];

% -- CT:
% - Lungs
% image_name      = [ 'l6', 'l7', 'l8' ]
% referencesToUse = [ 'l1', 'l1', 'l1' ]
% - Knee
% image_name      = [ 'rodilla2', 'rodilla4' ]
% referencesToUse = [ 'rodilla3', 'rodilla3' ]

% -- MRI:
% image_name      = [ 'sujet01_2_50', 'sujet03_2_70', 'sujet02_2_50' ]
% referencesToUse = [ 'sujet02_2_55', 'sujet04_2_70', 'sujet03_2_60' ]
%%

addpath('segmentationMethods\')

%% Program to check parameters of GAC and CV
% Input: array of images and array of parameters to test
% Output: txt files with the dice coefficient per image and combination of
% parameters, and the best combination of parameters found so far

SOURCE_FOLDER = 'images/source'; % folder where the target images are located
folder = './globalResults/';
OUTPUT_FOLDER = 'images/hippocampi';
REF_FOLDER = 'images/reference';
REF_CONT_FOLDER = 'images/reference_cont';
ELAB_FOLDER = 'images/elab';
ELAB_CONT_FOLDER = 'images/elab_cont';
FILENAME_TIME = 'images/execution_times.xlsx';
fileResultsLocalizationDG = 'localizationResultsDG';
fileResultsLocalizationCA = 'localizationResultsCA';
MODEL_TO_COMPARE = 'consensus'; % 'intersection','union','consensus','STAPLE'

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
All_ImageModality = {'Lungs'} %'Lungs' 'ISH'  'Knee' 'MRI'
All_Method_to_Tune = {'DSCV'} %'GAC' 'CV' 'DSGAC'
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for indexMethod = 1:size(All_Method_to_Tune,2)
    for indexModality = 1:size(All_ImageModality,2)
        ImageModality = char(All_ImageModality(indexModality));
        Method_to_Tune = char(All_Method_to_Tune(indexMethod));
        %% PARAMETERS
        countour_weight = [1 2 3];%[1 2 3 4 5];
        expansion_weight = [-1 -0.5];%[-2 -1 -0.5 0.5];
        
        if strcmp(ImageModality,'ISH')
            weightLengthTerm = [0.01 0.1 0.5];%[0.01 0.1 0.25 0.5 1];
        else
            weightLengthTerm = [0.01 0.1 0.25 0.75];%[0.01 0.1 0.25 0.5 1];
        end;
        lambda1 = [ 1, 2, 5];
        lambda2 = [ 1, 2, 5];
        % ADJUST THESE VALUES TO THE SIZE OF THE IMAGE! IF WE RESIZE THE IMAGES AND
        % WE WORK WITH THEM IN 500x500 WE NEED SMALLER FILTERS THAN 10x10 AND 20x20
        if strcmp(ImageModality,'ISH')
            sizeMedFilt = [5];%[5 10];%[5 10 15 20 25 30];
            sizeMedFiltGAC = [5 10];%[5 10 15 20 25 30];
        else
            sizeMedFilt = [1 3];
            sizeMedFiltGAC = [1 3];
        end;
        if strcmp(ImageModality,'ISH')
            minimumSize = 5000; % Half of the average size of the structures to segment.
        elseif strcmp(ImageModality,'Lungs')
            minimumSize = 200;%50;
            % CT: 25 structures to segment. Smallest regions: 1, 67, and 358 pixels.
            % We select 200, so we are able to segment 23 -> 92%
        elseif strcmp(ImageModality,'Knee')
            minimumSize = 200;%50;
        elseif strcmp(ImageModality,'MRI')
            minimumSize = 75;%1
            % MRI: 107 structures to segment. Smallest regions: 1, 6, 10, 6, 8, 3, 1
            % 3, 1, 1, 83, 98, 99. If we select 75 we are able to segment 97 -> 90%
        end;
        
        %% TRAINING IMAGES
        REFERENCES = [9; 10; 11; 12; 13; 14; 15; 16; 17; 18];
        if strcmp(ImageModality,'ISH')
            %     image_name = {'Zim1_117','Cutl2_30', 'Gfap_95_2307','Atrx_133_32'};
            %     referencesToUse = [16,11, 16,12];
            %             image_name = {'Wbp7_110','1300018I05Rik_86','0610010D24Rik_103_26','1300018I05Rik_94','5430437P03Rik_107_26',...
            %                 'Atrx_133_32','B230215L15Rik_129','Nmt1_140_34','Wars_109','Tubb3_114','Trem1_2800','3830406C13Rik_66',...
            %                 'A030009H04Rik_117','Atp1b2_2725','Azin1_96','Camk2a_102','Camk2b_102','Camk2g_128_2248','Cd9_30','Cutl2_30',...
            %                 'Gad1_104','Gapdh_93','Gfap_95_2307','Mbp_130','Slc17a7_120','Zim1_117'};
            %             referencesToUse = [15,14,10, 15,9,12,16,14,16,16,15,16, 14, 17, 17, 15,...
            %                 12, 17, 10, 11, 14, 12, 16, 17, 12, 16];
            image_name = {'0610010D24Rik_103_26','1300018I05Rik_86','1300018I05Rik_94','3830406C13Rik_66','5430437P03Rik_107_26',...
                'A030009H04Rik_117', 'Atp1b2_2725','Atrx_133_32','Azin1_96','B230215L15Rik_129','Camk2a_102','Camk2b_102',...
                'Camk2g_128_2248','Cd9_30','Cutl2_30','Gad1_104','Gapdh_93','Gfap_95_2307','Mbp_130','Nmt1_140_34','Slc17a7_120','Trem1_2800',...
                'Tubb3_114','Wars_109','Wbp7_110','Zim1_117' };
            referencesToUse = [10, 14, 15, 16, 10, 14, 17, 12, 17, 16, 15, 12, 17, 10,11,14,12,16, 17,14, 12, 15, 16,16,15 ,16  ];
        elseif strcmp(ImageModality,'MRI')
            %     image_name      = { 'sujet01_2_50', 'sujet03_2_70', 'sujet02_2_50'};
            %     REFERENCES = { 'sujet02_2_55'; 'sujet04_2_70';
            %     'sujet03_2_60' };
            image_name = {'sujet01_2_50', 'sujet01_2_55', 'sujet01_2_60','sujet02_2_50',...
                'sujet02_2_60','sujet03_2_65','sujet03_2_70','sujet04_2_60','sujet04_2_65',...
                'sujet05_2_55','sujet05_2_60','sujet05_2_65','sujet06_2_65','sujet06_2_70'};%,'sujet02_2_55', 'sujet04_2_70', 'sujet03_2_60'};
            REFERENCES = {'sujet02_2_55', 'sujet02_2_55', 'sujet04_2_70','sujet03_2_60',...
                'sujet04_2_70','sujet03_2_60','sujet04_2_70 ','sujet02_2_55','sujet02_2_55',...
                'sujet03_2_60','sujet02_2_55','sujet04_2_70','sujet02_2_55','sujet02_2_55'};%,'sujet02_2_55', 'sujet04_2_70', 'sujet03_2_60'};
            referencesToUse = [-1, -1, -1,-1, -1, -1,-1, -1, -1,-1, -1, -1,-1, -1];%,-1,-1,-1];
        elseif strcmp(ImageModality,'Lungs')
            image_name      = { 'l6', 'l7', 'l8'}%, 'lungs1', 'lungs2'};
            REFERENCES = {'l1', 'l1', 'l1'}%,'l1',  'l1' };
            referencesToUse = [-1, -1, -1]%,-1, -1];
        else
            image_name      = {'rodilla2', 'rodilla3', 'rodilla4' };
            REFERENCES = {'rodilla1', 'rodilla1', 'rodilla1'};
            referencesToUse = [-1, -1,-1];
        end;
        
        % image_name = {'Wbp7_110','1300018I05Rik_86','0610010D24Rik_103_26','1300018I05Rik_94','5430437P03Rik_107_26',...
        % 'Atrx_133_32','B230215L15Rik_129','Nmt1_140_34','Wars_109','Tubb3_114','Trem1_2800','3830406C13Rik_66',...
        % 'A030009H04Rik_117','Atp1b2_2725','Azin1_96','Camk2a_102','Camk2b_102','Camk2g_128_2248','Cd9_30','Cutl2_30',...
        % 'Gad1_104','Gapdh_93','Gfap_95_2307','Mbp_130','Slc17a7_120','Zim1_117'};
        % referencesToUse = [15,14,10, 15,9,12,16,14,16,16,15,16, 14, 17, 17, 15, 12, 17, 10, 11, 14, 12, 16, 17, 12, 16];
        
        commandsRun = {};
        Dices = [];
        DicePerCombination = [];
        
        timestamp = datestr(now, 'yymmdd_HHMMSS');
        out_dir_name = sprintf('%s/Tuning_%s_%s',OUTPUT_FOLDER, char(ImageModality),char(Method_to_Tune)); %timestamp);
        mkdir(out_dir_name);
        
        %% read images and save them in cell structures
        Images = {};
        for i = 1:size(image_name,2)
            hip_image_name = sprintf('%s/%s_ROI.png',SOURCE_FOLDER,char(image_name(i)));
            try
                Iorig = cvReadImage(hip_image_name,0);
            catch
                Iorig = imread(hip_image_name);
                if (size(Iorig,3)==3)
                    Iorig = rgb2gray(imread(source_image_name));
                end;
                disp('cvReadImage not available!');
            end;
            Images{i} = Iorig;
        end;
        
        GTs = {};
        for i = 1:size(image_name,2)
            hip_image_name = sprintf('%s/%s.mat',SOURCE_FOLDER,char(image_name(i)));
            load(hip_image_name);
            GTs{i} = new_image;
        end;
        
        if (strcmp(Method_to_Tune, 'DSGAC') | strcmp(Method_to_Tune, 'DSCV')) & strcmp(ImageModality,'MRI')
            bestPrior = {};
            listDir = dir('..\imagesCT_MRI_elastix\best_priors_mri\*.png');
            for i=1:size(listDir,1)
                priorImage = imread(strcat('..\imagesCT_MRI_elastix\best_priors_mri\',listDir(i).name));
                %                 priorImage(priorImage > 0)=1;
                bestPrior{i} = im2double(priorImage);
            end;
        end;
        if (strcmp(Method_to_Tune, 'DSGAC') | strcmp(Method_to_Tune, 'DSCV')) & strcmp(ImageModality,'ISH')
%             listDir = dir('..\ImagesCT_MRI_elastix\best_priors_ISH\best_results\*.mhd');
%             for i=1:size(listDir,1)
%                 command = ['elastixFiles\ConvertToPNG.exe ', strcat('..\ImagesCT_MRI_elastix\best_priors_ISH\best_results\', listDir(i).name)];
%                 system(command); %execute external command
%             end;

            bestPrior = {};
            listDir = dir('..\ImagesCT_MRI_elastix\best_priors_ISH\best_results\*.png');
            for i=1:size(listDir,1)
                priorImage = imread(strcat('..\ImagesCT_MRI_elastix\best_priors_ISH\best_results\',listDir(i).name));
                %                 priorImage(priorImage > 0)=1;
                bestPrior{i} = im2double(priorImage);
            end;
        end;
        if (strcmp(Method_to_Tune, 'DSGAC') | strcmp(Method_to_Tune, 'DSCV')) & strcmp(ImageModality,'Lungs')
%             listDir = dir('..\ImagesCT_MRI_elastix\best_prior_lungs\*.mhd');
%             for i=1:size(listDir,1)
%                 command = ['elastixFiles\ConvertToPNG.exe ', strcat('..\ImagesCT_MRI_elastix\best_prior_lungs\', listDir(i).name)];
%                 system(command); %execute external command
%             end;

            bestPrior = {};
            listDir = dir('..\ImagesCT_MRI_elastix\best_prior_lungs\*.png');
            for i=1:size(listDir,1)
                priorImage = imread(strcat('..\ImagesCT_MRI_elastix\best_prior_lungs\',listDir(i).name));
                %                 priorImage(priorImage > 0)=1;
                bestPrior{i} = im2double(priorImage);
            end;
        end;        
        if (strcmp(Method_to_Tune, 'DSGAC') | strcmp(Method_to_Tune, 'DSCV')) & strcmp(ImageModality,'Knee')
%             listDir = dir('..\ImagesCT_MRI_elastix\best_prior_knee\*.mhd');
%             for i=1:size(listDir,1)
%                 command = ['elastixFiles\ConvertToPNG.exe ', strcat('..\ImagesCT_MRI_elastix\best_prior_knee\', listDir(i).name)];
%                 system(command); %execute external command
%             end;

            bestPrior = {};
            listDir = dir('..\ImagesCT_MRI_elastix\best_prior_knee\*.png');
            for i=1:size(listDir,1)
                priorImage = imread(strcat('..\ImagesCT_MRI_elastix\best_prior_knee\',listDir(i).name));
                %                 priorImage(priorImage > 0)=1;
                bestPrior{i} = im2double(priorImage);
            end;
        end;  
        %% DSGAC
        if strcmp(Method_to_Tune, 'DSGAC')
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_GAC_out.txt'), 'w');
            index=1;
            Dices=[];
            DicePerCombinationMedian=[];
            DicePerCombinationMean = [];
            for h=1:size(countour_weight,2)
                for i=1:size(expansion_weight,2)
                    for j=1:size(sizeMedFiltGAC,2)
                        DicePerImage=[];
                        command = strcat(num2str(countour_weight(h)),'|', num2str(expansion_weight(i)), '|',num2str(sizeMedFiltGAC(j)));
                        fprintf(fid, 'Combination %s: \n', command);
                        for img=1:size(Images,2)
                            tic
                            if strcmp(ImageModality,'ISH')
                                [order best_ref] = sort(REFERENCES==referencesToUse(img), 'descend');
                                best_ref = best_ref(1);
                            else
                                best_ref = -1;
                            end;
                            [tout17, OutputImage] = segmentationGAC(im2double(Images{img}), ...
                                out_dir_name,image_name{img}, REFERENCES,best_ref,timestamp, bestPrior{img}, countour_weight(h), ...
                                expansion_weight(i), sizeMedFiltGAC(j),minimumSize,char(Method_to_Tune));
                            
                            % Read GT
                            GTImage = GTs{img};
                            
                            % Calculate Dice
                            x = find(OutputImage > 0);
                            y = find(GTImage > 0);
                            dice = 2*length(intersect(x, y)) / (length(x) + length(y));
                            
                            % Append file with command and
                            % dice result. Save also a cell
                            % array with commands and an
                            % array with Dices
                            
                            fprintf(fid, 'Image %d: %f\n', img, dice);
                            Dices = [Dices; dice];
                            commandsRun{index} = command;
                            index=index+1;
                            DicePerImage = [DicePerImage;dice];
                            toc
                        end;
                        DicePerCombinationMedian = [DicePerCombinationMedian;median(DicePerImage)];
                        DicePerCombinationMean = [DicePerCombinationMean;mean(DicePerImage)];
                    end;
                end;
            end;
            fclose('all');
            
            save(strcat(out_dir_name,filesep,'TuningGAC_',char(ImageModality),'.mat'),'Dices', 'DicePerCombinationMedian', 'DicePerCombinationMean', 'commandsRun')
            
            % Give visual output for the best Dice and best combination
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_GAC_FinalResults.txt'), 'w');
            [v in] = max(DicePerCombinationMedian)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Median Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            [v in] = max(DicePerCombinationMean)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Mean Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            fclose('all');
        end;
        
        %% GAC
        if strcmp(Method_to_Tune, 'GAC')
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_GAC_out.txt'), 'w');
            index=1;
            Dices=[];
            DicePerCombinationMedian=[];
            DicePerCombinationMean = [];
            for h=1:size(countour_weight,2)
                for i=1:size(expansion_weight,2)
                    for j=1:size(sizeMedFiltGAC,2)
                        DicePerImage=[];
                        command = strcat(num2str(countour_weight(h)),'|', num2str(expansion_weight(i)), '|',num2str(sizeMedFiltGAC(j)));
                        fprintf(fid, 'Combination %s: \n', command);
                        for img=1:size(Images,2)
                            tic
                            if strcmp(ImageModality,'ISH')
                                [order best_ref] = sort(REFERENCES==referencesToUse(img), 'descend');
                                best_ref = best_ref(1);
                            else
                                best_ref = -1;
                            end;
                            [tout17, OutputImage] = segmentationGAC(im2double(Images{img}), ...
                                out_dir_name,image_name{img}, REFERENCES,best_ref,timestamp, 0, countour_weight(h), ...
                                expansion_weight(i), sizeMedFiltGAC(j),minimumSize,char(Method_to_Tune));
                            
                            % Read GT
                            GTImage = GTs{img};
                            
                            % Calculate Dice
                            x = find(OutputImage > 0);
                            y = find(GTImage > 0);
                            dice = 2*length(intersect(x, y)) / (length(x) + length(y));
                            
                            % Append file with command and
                            % dice result. Save also a cell
                            % array with commands and an
                            % array with Dices
                            
                            fprintf(fid, 'Image %d: %f\n', img, dice);
                            Dices = [Dices; dice];
                            commandsRun{index} = command;
                            index=index+1;
                            DicePerImage = [DicePerImage;dice];
                            toc
                        end;
                        DicePerCombinationMedian = [DicePerCombinationMedian;median(DicePerImage)];
                        DicePerCombinationMean = [DicePerCombinationMean;mean(DicePerImage)];
                    end;
                end;
            end;
            fclose('all');
            
            save(strcat(out_dir_name,filesep,'TuningGAC_',char(ImageModality),'.mat'),'Dices', 'DicePerCombinationMedian', 'DicePerCombinationMean', 'commandsRun')
            
            % Give visual output for the best Dice and best combination
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_GAC_FinalResults.txt'), 'w');
            [v in] = max(DicePerCombinationMedian)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Median Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            [v in] = max(DicePerCombinationMean)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Mean Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            fclose('all');
        end;
        
        %% CV
        if strcmp(Method_to_Tune,'CV')
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_CV_out.txt'), 'w');
            index=1;
            Dices=[];
            DicePerCombinationMedian=[];
            DicePerCombinationMean = [];
            for i=1:size(weightLengthTerm,2)
                for j=1:size(sizeMedFilt,2)
                    for k=1:size(lambda1,2)
                        for l=1:size(lambda2,2)
                            DicePerImage=[];
                            command = strcat(num2str(weightLengthTerm(i)), '|',num2str(sizeMedFilt(j)), '|', num2str(lambda1(k)), '|',num2str(lambda2(l)));
                            fprintf(fid, 'Combination %s: \n', command);
                            for img=1:size(Images,2)
                                tic
                                if strcmp(ImageModality,'ISH')
                                    [order best_ref] = sort(REFERENCES==referencesToUse(img), 'descend');
                                    best_ref = best_ref(1);
                                else
                                    best_ref = -1;
                                end;
                                [tout17, OutputImage] = segmentationCV(im2double(Images{img}), ...
                                    out_dir_name,image_name{img}, SOURCE_FOLDER,REFERENCES,best_ref,timestamp, 0, weightLengthTerm(i), ...
                                    sizeMedFilt(j),lambda1(k),lambda2(l),minimumSize,char(Method_to_Tune));
                                
                                % Read GT
                                GTImage = GTs{img};
                                
                                % Calculate Dice
                                x = find(OutputImage > 0);
                                y = find(GTImage > 0);
                                dice = 2*length(intersect(x, y)) / (length(x) + length(y));
                                
                                % Append file with command and
                                % dice result. Save also a cell
                                % array with commands and an
                                % array with Dices
                                
                                fprintf(fid, 'Image %d: %f\n', img, dice);
                                Dices = [Dices; dice];
                                commandsRun{index} = command;
                                index=index+1;
                                DicePerImage = [DicePerImage;dice];
                                toc
                            end;
                            DicePerCombinationMedian = [DicePerCombinationMedian;median(DicePerImage)];
                            DicePerCombinationMean = [DicePerCombinationMean;mean(DicePerImage)];
                        end;
                    end;
                end;
            end;
            
            fclose('all');
            
            save(strcat(out_dir_name,filesep,'TuningCV_',char(ImageModality),'.mat'),'Dices', 'DicePerCombinationMedian', 'DicePerCombinationMean', 'commandsRun')
            
            % Give visual output for the best Dice and best combination
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_CV_FinalResults.txt'), 'w');
            [v in] = max(DicePerCombinationMedian)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Median Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            [v in] = max(DicePerCombinationMean)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Mean Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            fclose('all');
        end;
        
         %% DSCV
        if strcmp(Method_to_Tune,'DSCV')
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_DSCV_out.txt'), 'w');
            index=1;
            Dices=[];
            DicePerCombinationMedian=[];
            DicePerCombinationMean = [];
            for i=1:size(weightLengthTerm,2)
                for j=1:size(sizeMedFilt,2)
                    for k=1:size(lambda1,2)
                        for l=1:size(lambda2,2)
                            DicePerImage=[];
                            command = strcat(num2str(weightLengthTerm(i)), '|',num2str(sizeMedFilt(j)), '|', num2str(lambda1(k)), '|',num2str(lambda2(l)));
                            fprintf(fid, 'Combination %s: \n', command);
                            for img=1:size(Images,2)
                                tic
                                if strcmp(ImageModality,'ISH')
                                    [order best_ref] = sort(REFERENCES==referencesToUse(img), 'descend');
                                    best_ref = best_ref(1);
                                else
                                    best_ref = -1;
                                end;
                                [tout17, OutputImage] = segmentationCV(im2double(Images{img}), ...
                                    out_dir_name,image_name{img}, SOURCE_FOLDER,REFERENCES,best_ref,timestamp, bestPrior{img}, weightLengthTerm(i), ...
                                    sizeMedFilt(j),lambda1(k),lambda2(l),minimumSize,char(Method_to_Tune));
                                
                                % Read GT
                                GTImage = GTs{img};
                                
                                % Calculate Dice
                                x = find(OutputImage > 0);
                                y = find(GTImage > 0);
                                dice = 2*length(intersect(x, y)) / (length(x) + length(y));
                                
                                % Append file with command and
                                % dice result. Save also a cell
                                % array with commands and an
                                % array with Dices
                                
                                fprintf(fid, 'Image %d: %f\n', img, dice);
                                Dices = [Dices; dice];
                                commandsRun{index} = command;
                                index=index+1;
                                DicePerImage = [DicePerImage;dice];
                                toc
                            end;
                            DicePerCombinationMedian = [DicePerCombinationMedian;median(DicePerImage)];
                            DicePerCombinationMean = [DicePerCombinationMean;mean(DicePerImage)];
                        end;
                    end;
                end;
            end;
            
            fclose('all');
            
            save(strcat(out_dir_name,filesep,'TuningDSCV_',char(ImageModality),'.mat'),'Dices', 'DicePerCombinationMedian', 'DicePerCombinationMean', 'commandsRun')
            
            % Give visual output for the best Dice and best combination
            fid = fopen(strcat(out_dir_name,filesep,'CheckingParameters_DSCV_FinalResults.txt'), 'w');
            [v in] = max(DicePerCombinationMedian)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Median Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            [v in] = max(DicePerCombinationMean)
            commandsRun{(in-1)*size(Images,2)+1}
            fprintf(fid, 'Best Mean Dice value obtained: %f\n', v);
            fprintf(fid, 'Configuration used: %s\n', commandsRun{(in-1)*size(Images,2)+1});
            fclose('all');
        end;
    end;
end;
