#!/usr/bin/python
# Run elastix (2 phases), ofeli and validation

from subprocess import Popen, PIPE
import os
from os.path import join
from time import time, sleep
import itertools
import multiprocessing
from collections import namedtuple
from optparse import OptionParser
import re
import random
import glob
import shutil

# Read the configuration file
try:
    from config import *
except Exception, e:
    print "Configuration file incorrect or missing"
    print e
    exit()

Task = namedtuple('Task', 'fixed moving output_dir extconf_pOne extconf_pTwo extconf_Ofeli' )

class Writer(multiprocessing.Process):
    def __init__(self, results_queue, file_header):
        multiprocessing.Process.__init__(self)
        self.queue = results_queue
        self.file_header = file_header

    def run(self):
        results_file = open(results_file_name, 'w')
        results_file.write( self.file_header + "\n" )
        while True:
            try:
                res = self.queue.get()
            except KeyboardInterrupt, e:
                break
            if res is None:
                #Poison pill
                results_file.close()
                break
            results_file.write(res)
            results_file.flush()

class Executor(multiprocessing.Process):

    def __init__(self, task_queue, result_queue):
        multiprocessing.Process.__init__(self)
        self.task_queue = task_queue
        self.result_queue = result_queue

    def run(self):
        while True:
            task = self.task_queue.get()
            if task is None:
                # Poison pill means shutdown
                self.task_queue.task_done()
                break
            # Wait for a small random time, because of Elastix's poor random generator
            sleep(random.random())

            # Run elastix the first time # pOne
            start_time_pOne = time()

            args = [ elastix, '-f', join(input_data_path, task.fixed) + input_data_ext,
                     '-m', join(input_data_path, task.moving) + input_data_ext,
                     '-p', join(task.output_dir, 'pOne', params_file_name),
                     '-out', join(task.output_dir, 'pOne') ]

            if use_masks_fixed_pOne:
                args += [ '-fMask', join(masks_path, task.fixed) + '_mask' + masks_ext ]

            if use_masks_moving_pOne:
                args += [ '-mMask', join(masks_path, task.moving) + '_mask' + masks_ext ]

            if use_initial_transform:
                args += [ '-t0',
                          join(initial_transform_path,
                               "InitialTransform_{0}_{1}.txt".format( task.fixed, task.moving )) ]

            if options.verbose:
                proc = Popen(args)
            else:
                proc = Popen(args, stdout=PIPE, stderr=PIPE)
            try:
                out, err = proc.communicate()
            except KeyboardInterrupt, e:
                break
            except Exception, e:
                print e
                self.task_queue.task_done()
                continue

            if err != '' and err is not None:
                print 'Error while running Elastix for', task.fixed, 'and', task.moving
                print err
                self.task_queue.task_done()
                continue

            #Collect result of pOne
            timing_pOne = time() - start_time_pOne

            notfound = True
            with open(join(task.output_dir, 'pOne', 'elastix.log')) as res_file:
                for line in res_file.readlines():
                    if 'Final metric value' in line:
                        metric_value_pOne = float(line[22:])
                        notfound = False
                        break

                if notfound or metric_value_pOne == 0:
                    print "Error in the log file", join(task.output_dir, 'pOne', 'elastix.log')
                    self.task_queue.task_done()
                    continue

            args = [ validation_program, join(validation_data_path, task.fixed) + validation_data_ext,
                     join(validation_data_path, task.moving) + validation_data_ext,
                     "-t", join(task.output_dir, 'pOne', 'TransformParameters.0.txt'),
                     "-o", join(task.output_dir)]

            #keep validation output
            args += ["-s"]

            proc = Popen(args, stdout=PIPE, stderr=PIPE)
            try:
                out, err = proc.communicate()
            except Exception, e:
                print e
                self.task_queue.task_done()
                continue

            if err != '' and err is not None:
                print 'Error while performing validation for', task.fixed, 'and', task.moving
                print err
                self.task_queue.task_done()
                continue

            last_line = out.rstrip("\n").rpartition("\n")[2]
            validation_measure_pOne = float(last_line)

                ### pTwo ###
            # Run elastix the second time, combining the previous result
            start_time_pTwo = time()

            args = [ elastix, '-f', join(input_data_path, task.fixed) + input_data_ext,
                     '-m', join(input_data_path, task.moving) + input_data_ext,
                     '-p', join(task.output_dir, 'pTwo', params_file_name),
                     '-out', join(task.output_dir, 'pTwo'),
                     '-t0', join(task.output_dir, 'pOne', 'TransformParameters.0.txt') ]

            if use_masks_fixed_pTwo:
                args += [ '-fMask', join(masks_path, task.fixed) + '_mask' + masks_ext ]

            if use_masks_moving_pTwo:
                args += [ '-mMask', join(masks_path, task.moving) + '_mask' + masks_ext ]

            if options.verbose:
                proc = Popen(args)
            else:
                proc = Popen(args, stdout=PIPE, stderr=PIPE)
            try:
                out, err = proc.communicate()
            except KeyboardInterrupt, e:
                break
            except Exception, e:
                print e
                self.task_queue.task_done()
                continue

            if err != '' and err is not None:
                print 'Error while running Elastix for', task.fixed, 'and', task.moving
                print err
                self.task_queue.task_done()
                continue


            #Collect result
            timing_pTwo = time() - start_time_pTwo

            notfound = True
            with open(join(task.output_dir, 'pTwo', 'elastix.log')) as res_file:
                for line in res_file.readlines():
                    if 'Final metric value' in line:
                        metric_value = float(line[22:])
                        notfound = False
                        break

                if notfound or metric_value == 0:
                    print "Error in the log file", join(task.output_dir, 'pTwo', 'elastix.log')
                    self.task_queue.task_done()
                    continue

            args = [ validation_program, join(validation_data_path, task.fixed) + validation_data_ext,
                     join(validation_data_path, task.moving) + validation_data_ext,
                     "-t", join(task.output_dir, 'pTwo', 'TransformParameters.0.txt'),
                     "-o", join(task.output_dir)]

            #keep_validation_output:
            args += ["-s"]

            proc = Popen(args, stdout=PIPE, stderr=PIPE)
            try:
                out, err = proc.communicate()
            except Exception, e:
                print e
                self.task_queue.task_done()
                continue

            if err != '' and err is not None:
                print 'Error while performing validation for', task.fixed, 'and', task.moving
                print err
                self.task_queue.task_done()
                continue

            last_line = out.rstrip("\n").rpartition("\n")[2]
            validation_measure_pTwo = float(last_line)


            #Convert prior to PNG
            prior_img = "{0}_{1}".format(task.fixed, task.moving)
            args = [ converter, join(task.output_dir, prior_img) + '.mhd' ]
            proc = Popen(args)
            try:
                out, err = proc.communicate()
            except KeyboardInterrupt, e:
                break
            except Exception, e:
                print e
                print args
                exit()

            print "Conversion done"

            #Run Ofeli #######
            start_time_ofeli = time()

            args = [ ofeli_cmd, join(input_data_path, task.fixed) + ofeli_input_data_ext,
                     join(task.output_dir, prior_img) + ofeli_prior_data_ext, "-out_dir", join(task.output_dir, 'ofeli') ]

            for key in sorted(task.extconf_Ofeli.iterkeys()):
                args += [ key, str(task.extconf_Ofeli[key]) ]

            proc = Popen(args)
            try:
                out, err = proc.communicate()
            except KeyboardInterrupt, e:
                break
            except Exception, e:
                print e
                self.task_queue.task_done()
                continue

            if err != '' and err is not None:
                print 'Error while running for', task.fixed
                print err
                self.task_queue.task_done()
                continue

            print "Ofeli done"

            #Collect result
            timing_ofeli = time() - start_time_ofeli

            #The output is the "last" out file
            last_out_file = join(task.output_dir, 'ofeli', "out_last")

            args = [ validation_program, join(validation_data_path, task.fixed) + ofeli_validation_data_ext,
                     last_out_file + ofeli_output_data_ext ]
            proc = Popen(args, stdout=PIPE, stderr=PIPE)
            try:
                out, err = proc.communicate()
            except Exception, e:
                print 'An exception was raised when calling the validation program'
                print e
                self.task_queue.task_done()
                continue

            if err != '' and err is not None:
                print 'Error while performing validation for', task.fixed
                print err
                self.task_queue.task_done()
                continue

            validation_measure_ofeli = float(out)

            timing = time() - start_time_pOne

            # save results
            self.task_queue.task_done()

            # Prepare the output for the results file
            # Print the parameters that vary in the configurations
            # plus the ones in the report_paramters list

            temp_out = ""
            if task.extconf_pOne:
                temp_out += ", ".join( [ str(task.extconf_pOne[key])
                                     for key in sorted(task.extconf_pOne.iterkeys()) ] )
            if task.extconf_pTwo:
                temp_out += ", ".join( [ str(task.extconf_pTwo[key])
                                     for key in sorted(task.extconf_pTwo.iterkeys()) ] )
            if task.extconf_Ofeli:
                temp_out += ", ".join( [ str(task.extconf_Ofeli[key])
                                     for key in sorted(task.extconf_Ofeli.iterkeys()) ] )

            if reported_parameters:
                if task.extconf_pOne:
                    temp_out += ", "
                if task.extconf_pTwo:
                    temp_out += ", "
                if task.extconf_Ofeli:
                    temp_out += ", "

                temp = [ config_pOne[key] for key in sorted(reported_parameters) ] +\
                       [ config_pTwo[key] for key in sorted(reported_parameters) ]
                temp += [ config_Ofeli[key] for key in sorted(config_Ofeli.keys()) ]
                temp_out += ", ".join( temp )

            output_line = ", ".join(map(str, [ task.fixed, task.moving, temp_out, validation_measure_pOne, validation_measure_pTwo,
                                               validation_measure_ofeli, timing ])) + "\n"
            print validation_label+":", validation_measure_ofeli, "Time:", timing, \
              "Remaining:", self.task_queue.qsize()-1

            # # Rename the result image file, if present
            # for out_file in glob.glob(join(task.output_dir, 'pTwo', "result.0.*")):
            #     directory = os.path.split(out_file)[0]
            #     file_name, file_extension = os.path.splitext(out_file)
            #     new_name = "{0}_{1}".format(task.fixed, task.moving)
            #     os.rename( out_file, join( directory, new_name + file_extension) )
            #     # Change the header for mhd files
            #     if file_extension == ".mhd":
            #         with open(join(os.curdir, directory, new_name + file_extension)) as f:
            #             data = f.read()
            #         with open(join(os.curdir, directory, new_name + file_extension), "w") as f:
            #             f.write(re.sub("result.0", new_name, data) )

            # Remove the quotes symbols
            self.result_queue.put( output_line.replace( '"', '' ) )


options = None
config_pOne = default_config.copy()
config_pTwo = default_config.copy()
config_Ofeli = {}

if __name__ == '__main__':

    parser = OptionParser()
    parser.add_option("-v", "--verbose",
                      action="store_true", dest="verbose", default=False)
    (options, args) = parser.parse_args()

    if not os.path.exists(elastix):
        print "ERROR elastix not found. It is supposed to be in " + elastix
        exit()

    if not os.path.exists(ofeli_cmd):
        print "ERROR elastix not found. It is supposed to be in " + elastix
        exit()

    if not os.path.exists(input_data_path):
        print "ERROR Images not found, are you sure you are in the right directory?"
        exit()

    if not os.path.exists(validation_program):
        print "WARNING Validation program not found. It is supposed to be " + validation_program
        exit()

    if (use_masks_fixed_pOne or use_masks_moving_pOne or
        use_masks_fixed_pTwo or use_masks_moving_pTwo) and \
        not os.path.exists(masks_path):
        print "ERROR Masks not found. They are supposed to be in " + masks_path
        exit()

    if use_initial_transform and not os.path.exists(initial_transform_path):
        print "ERROR Initial transform not found. They are supposed to be in " + initial_transform_path
        exit()

    # Purge previous results
    for directory in glob.glob("results_*"):
        shutil.rmtree(directory)

    tasks = multiprocessing.JoinableQueue()
    results = multiprocessing.Queue()

    # Create the full configurations for the registration stages
    config_pOne.update(config_only_pOne)
    config_pTwo.update(config_only_pTwo)

    # Remove reported parameters that are already in variable parameters
    reported_parameters = filter(lambda x: x not in variable_parameters_pOne.keys() and x not in variable_parameters_pTwo.keys(),
                                 reported_parameters)

    # Create a list of dictionaries with all combination of values from variable_parameters
    extra_conf_pOne = [ dict( zip( variable_parameters_pOne.keys(), x ) )
                   for x in itertools.product( *variable_parameters_pOne.values() ) ]

    extra_conf_pTwo = [ dict( zip( variable_parameters_pTwo.keys(), x ) )
                   for x in itertools.product( *variable_parameters_pTwo.values() ) ]

    extra_conf_Ofeli = [ dict( zip( variable_parameters_Ofeli.keys(), x ) )
                   for x in itertools.product( *variable_parameters_Ofeli.values() ) ]

    # Start executors
    executors = [ Executor(tasks, results) for i in xrange(num_threads) ]
    for w in executors:
        w.start()


    for config_num_pOne, ext_pOne in enumerate(extra_conf_pOne):
        config_pOne.update(ext_pOne)
        for config_num_pTwo, ext_pTwo in enumerate(extra_conf_pTwo):
            config_pTwo.update(ext_pTwo)
            for config_num_Ofeli, ext_Ofeli in enumerate(extra_conf_Ofeli):
                config_Ofeli.update(ext_Ofeli)

                for (fixed, moving) in scenarios:
                    for rep in xrange(repetitions):
                        # Create the output directory
                        output_dir = "results_{0:04d}{1:04d}{2:06d}/{3}_vs_{4}_{5:04d}".format(config_num_pOne, config_num_pTwo, config_num_Ofeli, fixed, moving, rep)

                        if not os.path.exists(output_dir):
                            os.makedirs(join(output_dir, 'pOne'))
                            os.makedirs(join(output_dir, 'pTwo'))
                            os.makedirs(join(output_dir, 'ofeli'))

                        # Write the parameters files
                        with open(join(output_dir, 'pOne', params_file_name), 'w') as params_file:
                            for key in sorted(config_pOne.iterkeys()):
                                params_file.write("({0} {1})\n".format(key, config_pOne[key]))

                        with open(join(output_dir, 'pTwo', params_file_name), 'w') as params_file:
                            for key in sorted(config_pTwo.iterkeys()):
                                params_file.write("({0} {1})\n".format(key, config_pTwo[key]))

                    for rep in xrange(repetitions):
                        output_dir = "results_{0:04d}{1:04d}{2:06d}/{3}_vs_{4}_{5:04d}".format(config_num_pOne, config_num_pTwo, config_num_Ofeli, fixed, moving, rep)

                        #Run the test by calling Elastix, Ofeli and Validation
                        tasks.put(Task( fixed, moving, output_dir, extra_conf_pOne[config_num_pOne], extra_conf_pTwo[config_num_pTwo], extra_conf_Ofeli[config_num_Ofeli] ))

    # Add a poison pill for each executors
    for i in xrange( num_threads ):
        tasks.put(None)

    # Create results file header
    temp_out = ""
    if extra_conf_pOne[0]:
        temp_out += ", ".join(map(lambda x: 'pOne'+x , sorted(extra_conf_pOne[0].iterkeys())))
    if extra_conf_pTwo[0]:
        temp_out += ", ".join(map(lambda x: 'pTwo'+x, sorted(extra_conf_pTwo[0].iterkeys())))
    if extra_conf_Ofeli[0]:
        temp_out += ", ".join(map(lambda x: 'ofeli_'+x[1:], sorted(extra_conf_Ofeli[0].iterkeys())))

    if reported_parameters:
        if extra_conf_pOne[0]:
            temp_out += ", "
        if extra_conf_pTwo[0]:
            temp_out += ", "
        if extra_conf_Ofeli[0]:
            temp_out += ", "

        temp = [ 'pOne' + str(x) for x in sorted(reported_parameters) ] +\
               [ 'pTwo' + str(x) for x in sorted(reported_parameters) ]
        temp_out += ", ".join(temp)

    header = "fixed_img, moving_img, {0}, pOne.{1}, pTwo.{1}, ofeli.{1}, time".format(temp_out, validation_label)

    # Process that writes the output as soon as it is produced
    writer = Writer(results, header)
    writer.start()

    try:
        # Wait for all of the tasks to finish
        tasks.join()
        # Poison pill for the writer
        results.put(None)
        # Wait for the writer to finish
        writer.join()
    except KeyboardInterrupt, e:
        print "Interrupted by user"
        os._exit(1)



