
suppressMessages( library(doBy) )

rm(list = ls(all = TRUE))
.help.ESS <- help

input.file <- "results.csv"
validation.measure = "dice"

#Read the result file
results <- read.csv(file=input.file,
                    header=TRUE, stringsAsFactors = FALSE,
                    sep=",", strip.white=T)

config.fields <- names(results)[!names(results) %in% c("img",
                                                       validation.measure,
                                                       "time")]

# Baseline
baseline <- summaryBy(dice~img, data=subset(results, weightCV == 0 &
                              weightPrior == 0 & weightVFC == 0,
                              select=c(img, dice)), FUN=c(mean) )
names(baseline)[names(baseline) == "dice.mean"] <- "dice"
avg.baseline <- mean(baseline$dice)

# Overall results
groups <- summaryBy(formula( paste(validation.measure, " + time", "~", paste(config.fields, collapse = ' + '))),
                    data=results,
                    FUN=c(mean, sd) )

groups <- orderBy(~-dice.mean, data=groups)
best.groups <- groups[1:10,]
print("Overall")
print(paste("Baseline: ", avg.baseline))
print(best.groups)


# Per-image results
images = unique(results$img)
for (image in images)
{
  print(image)
  data <- subset(results, img == image)
  groups <- summaryBy(formula( paste(validation.measure, " + time", "~", paste(config.fields, collapse = ' + '))),
                      data=data,
                      FUN=c(mean, sd) )
  groups <- orderBy(~-dice.mean, data=groups)  
  best.groups <- groups[1:10,]
  baseline.img <- subset(baseline, img == image, select=c(dice))
  print(paste("baseline:", baseline.img))
  print(best.groups)
  print("")
}
