﻿/****************************************************************************
**
** Copyright (C) 2010-2011 Fabien Bessy.
** All rights reserved.
** Contact: fabien.bessy@gmail.com
**
** This file is part of Ofeli.
**
** You may use this file under the terms of the CeCILL license version 2 as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Fabien Bessy nor the names of its contributors
**     may be used to endorse or promote products derived from this
**     software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
**                    For all legal intents and purposes.
**
****************************************************************************/


//! \file main.cpp
//! \brief Ofeli
//! \author Fabien Bessy
//! \version 1.06
//! \date march 2012

/*! \mainpage Developer's documentation
 *
 * \section intro_sec Introduction
 *
 * <p>Ofeli, as an acronym for <b>O</b>pen, <b>F</b>ast and <b>E</b>fficient <b>L</b>evel set
   <b>I</b>mplementation, demonstrates how to operate an image
   segmentation algorithm of Y. Shi and W. C. Karl <b>[1],</b> using a
   discrete approach for the approximation of level-set-based
   curve evolution (implicit active contours).</p>
   <p>This is a novel (2005) and fast algorithm without the need
   of solving partial differential equations (PDE) while preserving the
   advantages of level set methods, such as the automatic handling of
   topological changes. Considerable speedups (×100) have been
   demonstrated as compared to PDE-based narrow band level-set implementations.</p>
   <p>The Home Page of Ofeli can be found at : http://code.google.com/p/ofeli/ .</p>

 * \section structure_sec Structure
 *
 * In this project, the <b>G</b>raphical <b>U</b>ser <b>I</b>nterface (GUI) is clearly separated of the image processing part.
 *
 * GUI part :
 * - file imageviewer.cpp file with the class ofeli::ImageViewer (dependant of the Qt4 framework)
 * - file pixmapwidget.cpp with the class ofeli::PixmapWidget (dependant of the Qt4 framework)
 *
 * Image processing part :
 * - file linked_list.cpp with the class ofeli::List
 * - file activecontour.cpp with the class ofeli::ActiveContour
 *      - file ac_withoutedges.cpp with the class ofeli::ACwithoutEdges
 *      - file ac_withoutedges_yuv.cpp with the class ofeli::ACwithoutEdgesYUV
 *      - file geodesic_ac.cpp with the class ofeli::GeodesicAC
 * - file hausdorff_distance.cpp with the class ofeli::HausdorffDistance
 * - file filters.cpp with the class ofeli::Filters (dependant of the Boost library for the random variables)
 * - file filters_rgb.cpp with class ofeli::FiltersRGB (dependant of the Boost library for the random variables)
 *
 *
 * \section reusability_sec Reusability
 *
 * <p>This Qt project is built as an application and not as a static or shared library. So if you are interested to use this c++ code, especially for the image processing part of the project,
 * you must just include this file(s) thanks to the preprocessor directive in your file(s). After you must just pass to each constructor an input argument pointer on a row-wise image data buffer and for the class ACwithoutEdgesYUV, a pointer on a RGB interleaved data buffer (R1 G1 B1 R2 G2 B2 ...).</p>
 * <p>If you prefer a command-line interface or if you are interested in a tracking example of this algorithm, you can find a fork of this project interfaced with Matlab (MEX-file). Each constructor take an input pointer on a column-wise image data buffer and for the class ACwithoutEdgesYUV, a pointer on a RGB planar data buffer (R1 R2 R3 ... G1 G2 G3 ... B1 B2 B3 ...).</p>
 *
 *
 * \section license_sec License
 *
 * This software is distributed under the <a href='http://www.cecill.info/licences/Licence_CeCILL_V2-en.html'>CeCILL license version 2</a> <a href='http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html'> (link to the french version here)</a>.
 *
 * \section acknowl_sec Acknowledgments
 *
 * Thanks to :
 * - J. Olivier, R. Boné, J-M. Girault, F. Amed, A. Lissy, C. Rouzière, L. Suta.
 * - <i>pattern recognition and image analysis research team</i>, <i>computer science laboratory</i>,
 * <i>François Rabelais University</i>.
 * - students and professors of the MSc in medical imaging.

<hr></hr>
* \section ref_sec References
<p><b>[1]</b> Y. Shi, W. C. Karl - <a href='https://docs.google.com/viewer?a=v&pid=explorer&chrome=true&srcid=0Bzx5IoqehNE_MGIwYmUwYzctYTRkMC00ODMwLWI3YmUtNTFjYThlMTBkOTIy&hl=en&authkey=CPT1xeYN'>A real-time algorithm for the approximation of level-set based curve evolution</a> - <i>IEEE Trans. Image Processing</i>, vol. 17, no. 5, May 2008</p>
<p><b>[2]</b> T. F. Chan, L. A. Vese - <a href='https://docs.google.com/viewer?a=v&pid=explorer&chrome=true&srcid=0Bzx5IoqehNE_NWY5ZGMyMmYtNzkwNi00NjI0LWE4ZGMtODllZTVmZWQ5NGRm&hl=en&authkey=CNfMkNEI'>Active contours without edges</a> - <i>IEEE Trans. Image Processing</i>, vol. 10, no. 2, Feb 2001</p>
<p><b>[3]</b> V. Caselles, R. Kimmel, G. Sapiro - <a href='https://docs.google.com/viewer?a=v&pid=explorer&chrome=true&srcid=0Bzx5IoqehNE_ZWEzNzk2ZjgtNzlkMi00NDY0LTkzZjQtYWQ5N2EyNDA5NGE3&hl=en&authkey=CKi1w7cE'>Geodesic active contours</a> - <i>International Journal of Computer Vision</i>, 22(1), 61–79 (1997)</p>
<p><b>[4]</b> P. Perona, J. Malik - <a href='https://docs.google.com/viewer?a=v&pid=explorer&chrome=true&srcid=0Bzx5IoqehNE_NmJmZWZkM2ItN2ZhZS00NjA4LTk3Y2UtNTNmYzkxYjFjNjU4&hl=en&authkey=CPDnxN8H'>Scale-space and edge detection using anistropic diffusion</a> - <i>IEEE Trans. Pattern Analysis and Machine Intelligence</i>, vol. 12, no. 17, Jul 1990</p>
 */

#include <QApplication>
#include <QCoreApplication>

#include "imageviewer.hpp"

int main(int argc, char *argv[])
{
    // to have the windows theme
    //QApplication::setStyle(new QWindowsStyle);

    QTextCodec::setCodecForLocale( QTextCodec::codecForName("UTF-8") );
    QTextCodec::setCodecForCStrings( QTextCodec::codecForName("UTF-8") );
    QTextCodec::setCodecForTr( QTextCodec::codecForName("UTF-8") );

    QApplication app(argc, argv);

    // to call the defaut constructor QSettings for the application in the class ImageViewer
    QCoreApplication::setOrganizationName("Bessy");
    QCoreApplication::setOrganizationDomain("bessy.com");
    QCoreApplication::setApplicationName("Ofeli");

    QSettings settings;
    int language = settings.value("language",0).toInt();

    QTranslator translator_qt_widget;
    QTranslator translator_ofeli;
    QString locale;

    switch( language )
    {
    case 0 :
    {
        locale = QLocale::system().name().section('_', 0, 0);
        break;
    }
    case 1 :
    {
        locale = QString("en");
        break;
    }
    case 2 :
    {
        locale = QString("fr");
        break;
    }
    default :
    {
        locale = QString("en");
    }
    }

    translator_qt_widget.load( QString("qt_") + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath) );
    app.installTranslator(&translator_qt_widget);

    translator_ofeli.load( QString(":ofeli_") + locale );
    app.installTranslator(&translator_ofeli);

    ofeli::ImageViewer imageViewer;
    imageViewer.show();

    return app.exec();
}
