#include <iostream>
#include <sstream>
#include <limits>
#include <fstream>
#include <string>
#include <vector>
#include <math.h>
#include <fftw3.h>
#include <stdio.h>

#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/gpu/gpu.hpp"
#include "vfc.h"

using namespace std;

string basepath;
string svf_fname;

fftw_complex *in_k, *out1_k, *out2_k, *out3_k;
fftw_plan p1_k, p2_k, p3_k;

fftw_complex *in_i, *out1_i, *out2_i, *out3_i;
fftw_plan p1_i, p2_i, p3_i;

int n0_k, n1_k, n0_i, n1_i;
int r, Ksize;

// void
// init_plans (int img_width, int img_height)
// {
//   r = std::max (img_width, img_height) / 2 + 1;
//   Ksize = 2 * r + 1;

//   n0_k = Ksize + img_width - 1;
//   n1_k = Ksize + img_height - 1;
//   if (n1_k < n0_k)
//     n1_k = n0_k;
//   else
//     n0_k = n1_k;

//   // Planning-rigor flags
//   // FFTW_ESTIMATE
//   // FFTW_MEASURE
//   // FFTW_PATIENT
//   // FFTW_EXHAUSTIVE

//   in_k = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_k * n1_k);
//   out1_k = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_k * n1_k);
//   out2_k = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_k * n1_k);
//   out3_k = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_k * n1_k);
//   p1_k =
//     fftw_plan_dft_2d (n0_k, n1_k, in_k, out1_k, FFTW_FORWARD, FFTW_ESTIMATE);
//   p2_k =
//     fftw_plan_dft_2d (n0_k, n1_k, in_k, out2_k, FFTW_FORWARD, FFTW_ESTIMATE);
//   p3_k =
//     fftw_plan_dft_2d (n0_k, n1_k, in_k, out3_k, FFTW_BACKWARD, FFTW_ESTIMATE);


//   n0_i = img_width + img_width - 1;
//   n1_i = img_height + img_height - 1;
//   if (n1_i < n0_i)
//     n1_i = n0_i;
//   else
//     n0_i = n1_i;

//   in_i = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_i * n1_i);
//   out1_i = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_i * n1_i);
//   out2_i = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_i * n1_i);
//   out3_i = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0_i * n1_i);
//   p1_i =
//     fftw_plan_dft_2d (n0_i, n1_i, in_i, out1_i, FFTW_FORWARD, FFTW_ESTIMATE);
//   p2_i =
//     fftw_plan_dft_2d (n0_i, n1_i, in_i, out2_i, FFTW_FORWARD, FFTW_ESTIMATE);
//   p3_i =
//     fftw_plan_dft_2d (n0_i, n1_i, in_i, out3_i, FFTW_BACKWARD, FFTW_ESTIMATE);
// }

// // save a dmat to a binary file
void
save_vfc_component (string file, dmat & m)
{
  // Initialisation des variables locales
  int w;
  int h = m.size ();
  if (h)
    w = m[0].size ();
  else
    return;

  string fname = basepath + file;
  ofstream ofs( fname.c_str(), ios::binary | ios::out);

  double val;
  int i, j;
  for (i = 0; i < h; i++)
    {
      for (j = 0; j < w; j++)
      {
        val = m[i][j];
        ofs.write( reinterpret_cast<char*>( &val ), sizeof(double) );
      }
    }

  ofs.close();
}

// read a dmat from binary file
void
read_vfc_component (string filename, dmat & m, int width, int height)
{
  init (m, width, height);

  std::ifstream file(filename.c_str());
  int i, j;
  double val;
  for (i = 0; i < height; i++)
    {
      for (j = 0; j < width; j++)
      {
        file.read( reinterpret_cast<char*>( &val ), sizeof(double) );
        m[i][j] = val;
      }
    }

  file.close();
}

// this function saves two .csv files for x and y coordinates of
// a vector field. an R script will read this and plot a pdf. the title
// of the pdf and the names of the csv are saved to a text file so that
// the R script will plot all the pdfs in an automatic way.
void
save_vf (string fname, string title, dmat & x, dmat & y)
{
  static int seqnr = 0;
  string filename;
  ostringstream oss;
  oss << ++seqnr << "-" << fname;
  filename = oss.str ();

  string namex = filename + "_x.csv";
  string namey = filename + "_y.csv";
  save_vfc_component (namex, x);
  save_vfc_component (namey, y);

  string entry = namex + "," + namey + "," + title + "\n";

  FILE *fp = fopen (svf_fname.c_str (), "a");
  fprintf (fp, entry.c_str ());
  fclose (fp);
}

// // print a dmat on screen
// void
// print (dmat const &v)
// {
//   int w;
//   int h = v.size ();
//   if (h)
//     w = v[0].size ();
//   else
//     return;

//   for (int i = 0; i < h; ++i)
//     {
//       for (int j = 0; j < w; ++j)
// 	cout << v[i][j] << " ";
//       cout << endl;
//     }
//   cout << endl;
// }

// inits a dmat giving the two dimensions
void
init (dmat & v, int w, int h)
{
  v.clear ();

  for (int i = 0; i < h; ++i)
    {
      v.push_back (vector < double >());
      v[i].resize (w);
    }
}


// calculating the convolution of the kernel and the image.
// we do it in the frequency domain as a multiplication, so:
// we calculate the fft of the kernel and the input image,
// we multiply them in the frequency domain, then
// we reverse fft the result in the image domain and
// we remove the padded points.
// the padded points are necessary for the convolution.
void
convolute (dmat const &kernelX,
	   dmat const &kernelY, dmat const &img, dmat & rX, dmat & rY)
{
  // image size
  int w;
  int h = img.size ();
  if (h)
    w = img[0].size ();
  else
    return;

  // kernel size
  int kw;
  int kh = kernelX.size ();
  if (kh)
    kw = kernelX[0].size ();
  else
    return;

  //cout << "w= " << w << " h= " << h << " kw= " << kw << " kh= " << kh << endl;


  //static fftw_complex *in, *out1, *out2, *out3;
  //static fftw_plan p1,p2,p3;
  fftw_complex *in, *out1, *out2, *out3;
  fftw_plan p1, p2, p3;

  // calculating the size with the padded points
  int n0 = kw + w - 1;
  int n1 = kh + h - 1;
  if (n1 < n0)
    n1 = n0;
  else
    n0 = n1;

  in = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0 * n1);
  out1 = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0 * n1);
  out2 = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0 * n1);
  out3 = (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * n0 * n1);
  p1 = fftw_plan_dft_2d (n0, n1, in, out1, FFTW_FORWARD, FFTW_ESTIMATE);
  p2 = fftw_plan_dft_2d (n0, n1, in, out2, FFTW_FORWARD, FFTW_ESTIMATE);
  p3 = fftw_plan_dft_2d (n0, n1, in, out3, FFTW_BACKWARD, FFTW_ESTIMATE);


  // copy data in inputs (kernel)
  for (int i = 0; i < n1; ++i)
    {
      for (int j = 0; j < n0; ++j)
	{
	  if (i < kh && j < kw)
	    {
	      in[j + i * n0][0] = kernelX[i][j];
	      in[j + i * n0][1] = kernelY[i][j];
	    }
	  else
	    {
	      in[j + i * n0][0] = 0;
	      in[j + i * n0][1] = 0;
	    }
	}
    }



  // fft of the kernel
  fftw_execute (p1);

  // copy data in inputs (image)
  for (int i = 0; i < n1; ++i)
    {
      for (int j = 0; j < n0; ++j)
	{
	  if (i < h && j < w)
	    in[j + i * n0][0] = img[i][j];
	  else
	    in[j + i * n0][0] = 0;

	  in[j + i * n0][1] = 0;
	}
    }

  // fft of the image
  fftw_execute (p2);

  // multiplication in frequency domain
  // (a+bi)*(c+di) = (ac-bd) + (ad+bc)i
  for (int i = 0; i < n0 * n1; ++i)
    {
      // real
      in[i][0] = out1[i][0] * out2[i][0] - out1[i][1] * out2[i][1];
      // imag
      in[i][1] = out1[i][0] * out2[i][1] + out1[i][1] * out2[i][0];
    }

  // reverse fft of the result of the multiplication
  fftw_execute (p3);

  // initialize results arrays
  init (rX, w, h);
  init (rY, w, h);

  //remove padded points
  for (int i = kh / 2; i < kh / 2 + h; ++i)
    for (int j = kw / 2; j < kw / 2 + w; ++j)
      {
	rX[i - kh / 2][j - kw / 2] = out3[j + i * n0][0] / (n0 * n1);
	rY[i - kh / 2][j - kw / 2] = out3[j + i * n0][1] / (n0 * n1);
      }

  //cout << "w= " << w << " h= " << h << " kw= " << kw << " kh= " << kh
  //               << " n0= " << n0 << " n1 = " << n1 << endl;

  // cleanup of the fftw3 structures
  fftw_destroy_plan (p1);
  fftw_destroy_plan (p2);
  fftw_destroy_plan (p3);
  fftw_free (in);
  fftw_free (out1);
  fftw_free (out2);
  fftw_free (out3);
}

void
convolute (dmat const &kernelX,
	   dmat const &kernelY,
	   dmat const &img,
	   dmat & rX,
	   dmat & rY,
	   fftw_complex * in,
	   fftw_complex * out1,
	   fftw_complex * out2,
	   fftw_complex * out3,
	   fftw_plan p1, fftw_plan p2, fftw_plan p3, int n0, int n1)
{
  // image size
  int w;
  int h = img.size ();
  if (h)
    w = img[0].size ();
  else
    return;

  // kernel size
  int kw;
  int kh = kernelX.size ();
  if (kh)
    kw = kernelX[0].size ();
  else
    return;

  //cout << "w= " << w << " h= " << h << " kw= " << kw << " kh= " << kh << endl;


  // copy data in inputs (kernel)
  for (int i = 0; i < n1; ++i)
    {
      for (int j = 0; j < n0; ++j)
	{
	  if (i < kh && j < kw)
	    {
	      in[j + i * n0][0] = kernelX[i][j];
	      in[j + i * n0][1] = kernelY[i][j];
	    }
	  else
	    {
	      in[j + i * n0][0] = 0;
	      in[j + i * n0][1] = 0;
	    }
	}
    }

  // fft of the kernel
  fftw_execute (p1);

  // copy data in inputs (image)
  for (int i = 0; i < n1; ++i)
    {
      for (int j = 0; j < n0; ++j)
	{
	  if (i < h && j < w)
	    in[j + i * n0][0] = img[i][j];
	  else
	    in[j + i * n0][0] = 0;

	  in[j + i * n0][1] = 0;
	}
    }

  // fft of the image
  fftw_execute (p2);

  // multiplication in frequency domain
  // (a+bi)*(c+di) = (ac-bd) + (ad+bc)i
  for (int i = 0; i < n0 * n1; ++i)
    {
      // real
      in[i][0] = out1[i][0] * out2[i][0] - out1[i][1] * out2[i][1];
      // imag
      in[i][1] = out1[i][0] * out2[i][1] + out1[i][1] * out2[i][0];
    }

  // reverse fft of the result of the multiplication
  fftw_execute (p3);

  // initialize results arrays
  init (rX, w, h);
  init (rY, w, h);

  //remove padded points
  for (int i = kh / 2; i < kh / 2 + h; ++i)
    for (int j = kw / 2; j < kw / 2 + w; ++j)
      {
	rX[i - kh / 2][j - kw / 2] = out3[j + i * n0][0] / (n0 * n1);
	rY[i - kh / 2][j - kw / 2] = out3[j + i * n0][1] / (n0 * n1);
      }

  //cout << "w= " << w << " h= " << h << " kw= " << kw << " kh= " << kh
  //               << " n0= " << n0 << " n1 = " << n1 << endl;
}

void
convolute_k (dmat const &kernelX,
	     dmat const &kernelY, dmat const &img, dmat & rX, dmat & rY)
{
  convolute (kernelX, kernelY, img, rX, rY,
	     in_k, out1_k, out2_k, out3_k, p1_k, p2_k, p3_k, n0_k, n1_k);
}

void
convolute_i (dmat const &kernelX,
	     dmat const &kernelY, dmat const &img, dmat & rX, dmat & rY)
{
  convolute (kernelX, kernelY, img, rX, rY,
	     in_i, out1_i, out2_i, out3_i, p1_i, p2_i, p3_i, n0_i, n1_i);
}

//this function builds the standard vector field convolution kernel
void
buildVFK (dmat & x, dmat & y, int r, double a)
{
  int Ksize = 2 * r + 1;

  //dmat dist;
  double dist;
  init (x, Ksize, Ksize);
  init (y, Ksize, Ksize);
  //init(dist, Ksize, Ksize);


  for (int i = 0; i < 2 * r + 1; ++i)
    for (int j = 0; j < 2 * r + 1; ++j)
      {
	x[i][j] = r - j;
	y[i][j] = r - i;

	dist = sqrt (x[i][j] * x[i][j] + y[i][j] * y[i][j]);
	if (dist > r)
	  {
	    x[i][j] = y[i][j] = 0;
	  }
	else
	  {
	    dist = pow (dist, a + 1);
	    if (dist != 0)
	      {
		x[i][j] /= dist;
		y[i][j] /= dist;
	      }
	    else
	      {
		x[i][j] = y[i][j] = 0;
	      }
	  }
      }
}

// this function uses Opencv 2.3 to read the images, to apply
// some filtering to them, and to generate the gradient images
// and vector fields (obtained by the horiz and vert sobel).
int
generateImages (		//string name,
		 cv::Mat & src,
		 cv::Mat & filtered,
		 cv::Mat & grad_x,
		 cv::Mat & grad_y,
		 cv::Mat & grad,
		 double &otsu_th, double sigma1, double sigma2,
                 bool verbose)
{

  if (!src.data)
    {
      return -1;
    }

  /// Filtering
  cv::GaussianBlur (src, filtered, cv::Size (3, 3), sigma1 /*sigma */ , 0,
		    cv::BORDER_DEFAULT);
  if (verbose)
  {
    string fname = basepath + "filtered.jpg";
    cv::imwrite (fname, filtered);
  }


  int scale = 1;
  int delta = 0;
  int ddepth = CV_64F;
  //int ddepth = CV_16S;



  cv::GaussianBlur (src, src, cv::Size (3, 3), sigma2 /*sigma */ , 0,
		    cv::BORDER_DEFAULT);

  /// Generate grad_x and grad_y
  cv::Mat abs_grad_x, abs_grad_y;

  /// Gradient X
  //Scharr( src_gray, grad_x, ddepth, 1, 0, scale, delta, BORDER_DEFAULT );
  cv::Sobel (src, grad_x, ddepth, 1, 0, 3, scale, delta, cv::BORDER_DEFAULT);
  cv::convertScaleAbs (grad_x, abs_grad_x);

  /// Gradient Y
  //Scharr( src_gray, grad_y, ddepth, 0, 1, scale, delta, BORDER_DEFAULT );
  cv::Sobel (src, grad_y, ddepth, 0, 1, 3, scale, delta, cv::BORDER_DEFAULT);
  cv::convertScaleAbs (grad_y, abs_grad_y);

  /// Total Gradient (approximate)
  cv::addWeighted (abs_grad_x, 0.5, abs_grad_y, 0.5, 0, grad);


  int erosion_size = 1;
  cv::Mat element = cv::getStructuringElement (cv::MORPH_CROSS,
					       cv::Size (2 * erosion_size + 1,
							 2 * erosion_size +
							 1),
					       cv::Point (erosion_size,
							  erosion_size));
  cv::Mat orig_grad = grad.clone ();
  cv::erode (grad, grad, element);

  // save gradient images
//      fname = basepath + "grad_orig.jpg";
//      cv::imwrite(fname, orig_grad);
//      fname = basepath + "grad_x.jpg";
//      cv::imwrite(fname, grad_x);
//      fname = basepath + "grad_y.jpg";
//      cv::imwrite(fname, grad_y);
  // fname = basepath + "gradientOriginal.pgm";
  // cv::imwrite (fname, grad);


  // calculating otsu's thresold
  cv::Mat img_otsu;
  otsu_th = cv::threshold (grad, img_otsu, 100, 100, cv::THRESH_OTSU);

  if (verbose)
  {
    string fname = basepath + "otsu.jpg";
    cv::imwrite (fname, img_otsu);
  }

  return 1;
}

void
dmatToCvMat (dmat const &f, cv::Mat & img)
{
  int w;
  int h = f.size ();
  if (h)
    w = f[0].size ();
  else
    return;

  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      img.at < unsigned char >(i, j) = static_cast < unsigned char >(f[i][j]);
}

// copy a cv::Mat to dmat
void
clone (dmat & f, cv::Mat & img)
{
  int h = img.rows;
  int w = img.cols;

  init (f, w, h);

  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      f[i][j] = static_cast < double >(img.at < double >(i, j));
}

// copy a cv::Mat to dmat
void
cloneUint (dmat & f, cv::Mat & img)
{
  int h = img.rows;
  int w = img.cols;

  init (f, w, h);

  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      f[i][j] = static_cast < double >(img.at < unsigned char >(i, j));
}

// clone a dmat
void
clone (dmat & dst, const dmat & src)
{
  int w;
  int h = src.size ();
  if (h)
    w = src[0].size ();
  else
    return;

  init (dst, w, h);

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      dst[i][j] = src[i][j];
}

// normalization in the range [0,1]
void
normalize01 (dmat & f, cv::Mat & img)
{
  int h = img.rows;
  int w = img.cols;

  init (f, w, h);
  double min = numeric_limits < double >::max (),
    max = numeric_limits < double >::min ();

  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      {
	f[i][j] = static_cast < double >(img.at < unsigned char >(i, j));

	if (f[i][j] > max)
	  max = f[i][j];
	if (f[i][j] < min)
	  min = f[i][j];
      }

  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      f[i][j] = (f[i][j] - min) / max;
}

void
saveDmatAsImage (string name, dmat const &f, double factor = 255)
{
  int w;
  int h = f.size ();
  if (h)
    w = f[0].size ();
  else
    return;

  // saving anisotropic term as an image
  string fname = basepath + name;
  cv::Mat img = cv::Mat (h, w, CV_8UC1);
  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      img.at < unsigned char >(i, j) =
    static_cast < unsigned char >(f[i][j] * factor);
  cv::imwrite (fname, img);
}

// inverts an image in the 01 range
void
invert01 (dmat & f)
{
  int w;
  int h = f.size ();
  if (h)
    w = f[0].size ();
  else
    return;

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      f[i][j] = 1.0 - f[i][j];
}

// normalization of all the vector magnitudes to length param
void
normalizeLength (dmat & u, dmat & v, double length = 1.0)	//length is the scaling factor
{
  int w;
  int h = u.size ();
  if (h)
    w = u[0].size ();
  else
    return;

  double eps = numeric_limits < double >::epsilon ();	// smallest positive number
  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	double mod = sqrt (u[i][j] * u[i][j] + v[i][j] * v[i][j]) + eps;
	u[i][j] = (u[i][j] / mod) * length;
	v[i][j] = (v[i][j] / mod) * length;
      }
}

// this function builds the kernel using the anisotropic term
void
buildVFK_anisotropic (dmat & u, dmat & v,	// displacement map
		      dmat & m1x, dmat & m1y,	// kernel
		      double a)
{
  // kernel size
  int w;
  int h = u.size ();
  if (h)
    w = u[0].size ();
  else
    return;

  double ndist, c1;

  init (m1x, w, h);
  init (m1y, w, h);

  double eps = numeric_limits < double >::epsilon ();	// smallest positive number
  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      {
	m1x[i][j] = w / 2 - j;
	m1y[i][j] = h / 2 - i;
	ndist = sqrt (m1x[i][j] * m1x[i][j] + m1y[i][j] * m1y[i][j]) + eps;
	m1x[i][j] /= ndist;
	m1y[i][j] /= ndist;

	c1 = 1 / (2 - (m1x[i][j] * u[i][j] + m1y[i][j] * v[i][j]));

	m1x[i][j] = w / 2 - j;
	m1y[i][j] = h / 2 - i;
	ndist = pow (ndist, a + 1);
	m1x[i][j] /= ndist;
	m1y[i][j] /= ndist;

	m1x[i][j] *= c1;
	m1y[i][j] *= c1;
	if (ndist == 0)
	  {
	    m1x[i][j] = 0;
	    m1y[i][j] = 0;
	  }
      }


  //save_vf("m1", "Anisotropic kernel vector field", m1x, m1y);


  // saving anisotropic term
  //saveDmatAsImage("anis.png", c1);
}


// test function
void
save_N_kernelTerm (int w, int h)
{
  dmat testx, testy, ndist;
  init (testx, w, h);
  init (testy, w, h);
  init (ndist, w, h);

  double eps = numeric_limits < double >::epsilon ();	// smallest positive number
  for (int i = 0; i < h; ++i)
    for (int j = 0; j < w; ++j)
      {
	testx[i][j] = w / 2 - j;
	testy[i][j] = h / 2 - i;
	ndist[i][j] =
	  sqrt (testx[i][j] * testx[i][j] + testy[i][j] * testy[i][j]) + eps;
	testx[i][j] /= ndist[i][j];
	testy[i][j] /= ndist[i][j];
      }


  save_vf ("n", "Unit vector field", testx, testy);
}


// this way we store the information about different lengths of every vector.
// the mean length of the vfc modules is imposed = 1. since we are interested in having
// the mean length of the mix modules = 1, we normalize the length of the
// "gradient" modules imposing the mean of the vectors whose modules are > than
// the otsu's threshold = to the corresponding vectors in the vfc field.
void
normalizeMeans (dmat & u,
		dmat & v,
		dmat & grad_x,
		dmat & grad_y, dmat const &grad, double otsu_th)
{
  int w;
  int h = u.size ();
  if (h)
    w = u[0].size ();
  else
    return;

  double mean_mod_vfc = 0;

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      mean_mod_vfc += sqrt (u[i][j] * u[i][j] + v[i][j] * v[i][j]);
  mean_mod_vfc /= (w * h);

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	u[i][j] /= mean_mod_vfc;
	v[i][j] /= mean_mod_vfc;
      }

  double mean_vfc_otsu = 0, mean_grad_otsu = 0, count_otsu = 0;
  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	if (grad[i][j] > otsu_th)
	  {
	    mean_grad_otsu += sqrt (grad_x[i][j] * grad_x[i][j] +
				    grad_y[i][j] * grad_y[i][j]);
	    mean_vfc_otsu += sqrt (u[i][j] * u[i][j] + v[i][j] * v[i][j]);
	    count_otsu++;
	  }
      }
  mean_grad_otsu /= count_otsu;
  mean_vfc_otsu /= count_otsu;
  mean_grad_otsu /= mean_vfc_otsu;

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	grad_x[i][j] /= mean_grad_otsu;
	grad_y[i][j] /= mean_grad_otsu;
      }
}

// mixing the VFC and Gradient. the mix field is = to the vfc field but,
// where the modulus of the gradient field is > than the otsu's threshold
// we substitute the vfc with the gradient
void
mixOtsu (dmat & mix_u,
	 dmat & mix_v,
	 dmat const &u,
	 dmat const &v,
	 dmat const &gx, dmat const &gy, dmat const &gr, double otsu_th)
{
  int w;
  int h = gx.size ();
  if (h)
    w = gx[0].size ();
  else
    return;

  init (mix_u, w, h);
  init (mix_v, w, h);

  double count_mix = 0;
  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	if (gr[i][j] > otsu_th)
	  {
	    mix_u[i][j] = gx[i][j];
	    mix_v[i][j] = gy[i][j];
	    count_mix++;
	  }
	else
	  {
	    mix_u[i][j] = u[i][j];
	    mix_v[i][j] = v[i][j];
	  }
      }

  cout << "mix: " << count_mix / (w * h) *
    100 << " % pixels from gradient" << endl;
}


// convex combination between the VFC and gradient.
// smooth transition between the two.
void
mixLinearCombination (dmat & mix_u,
		      dmat & mix_v,
		      dmat const &u,
		      dmat const &v,
		      dmat const &gx,
		      dmat const &gy, dmat const &gr, double K)
{
  int h = u.size ();
  int w = u[0].size ();

  dmat g;
  init (g, w, h);
  init (mix_u, w, h);
  init (mix_v, w, h);

  double mean_g = 0;
  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	g[i][j] = exp (-pow (gr[i][j] / K, 2));
	mean_g += g[i][j];

	mix_u[i][j] = (1.0 - g[i][j]) * gx[i][j] + g[i][j] * u[i][j];
	mix_v[i][j] = (1.0 - g[i][j]) * gy[i][j] + g[i][j] * v[i][j];
      }

  mean_g /= (w * h);
  cout << "mix LC, mean factor for u,v = " << mean_g << endl;

  //saveDmatAsImage("g_function.pgm", g);
}

// inverts a field in the sense that we calculate the modulus of the largest vector, |l|
// than for each vector v we calculate its new lenght as "|l| - |v|" so that
// the long vectors are short and vice versa
void
invertField (dmat & u, dmat & v)
{
  int w;
  int h = u.size ();
  if (h)
    w = u[0].size ();
  else
    return;


  double longest = 0;

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	double l = sqrt (u[i][j] * u[i][j] + v[i][j] * v[i][j]);
	if (l > longest)
	  longest = l;
      }

  for (int i = 0; i < h; i++)
    for (int j = 0; j < w; j++)
      {
	double l = sqrt (u[i][j] * u[i][j] + v[i][j] * v[i][j]);
	double diff = longest - l;
	double factor;
	if (l != 0)
	  {
	    factor = diff / l;
	    u[i][j] = u[i][j] * factor;
	    v[i][j] = v[i][j] * factor;
	  }

      }
}


// void
// getVFC_DistanceGradient (dmat & VFC_DG,
// 			 dmat const &u,
// 			 dmat const &v,
// 			 cv::Mat & img,
// 			 int nsize,
// 			 int equalize,
// 			 double vfcConst,
// 			 bool filterVFC_DG, bool saveImage = true)
// {
//   int h = u.size ();
//   int w = u[0].size ();
//   init (VFC_DG, w, h);

//   double min = numeric_limits < double >::max (),
//     max = numeric_limits < double >::min ();


//   double MagSum;
//   for (int i = 0; i < h; i++)
//     for (int j = 0; j < w; j++)
//       {

// 	MagSum = 0;
// 	for (int y = -nsize; y <= nsize; y++)
// 	  for (int x = -nsize; x <= nsize; x++)
// 	    {
// 	      int idy = (i + y) % h;
// 	      if (idy < 0)
// 		idy += h;	// c++ does not wrap around negative numbers in modulo
// 	      int idx = (j + x) % w;
// 	      if (idx < 0)
// 		idx += w;

// 	      MagSum += sqrt (pow (u[idy][idx], 2) + pow (v[idy][idx], 2));

// 	    }

// 	if (MagSum < 0)
// 	  MagSum = 0.0001;
// 	VFC_DG[i][j] = 1 / MagSum;


// 	// this is done for normalization
// 	if (VFC_DG[i][j] > max)
// 	  max = VFC_DG[i][j];
// 	if (VFC_DG[i][j] < min)
// 	  min = VFC_DG[i][j];
//       }

//   double min2 = numeric_limits < double >::max (),
//     max2 = numeric_limits < double >::min ();
//   // normalization in [0, 65536]
//   for (int i = 0; i < h; ++i)
//     for (int j = 0; j < w; ++j)
//       {
// 	//VFC_DG[i][j] = (( VFC_DG[i][j] - min ) / max) * 255.0 * 255.0;
// 	VFC_DG[i][j] =
// 	  (VFC_DG[i][j] * 65535.0) / (max - min) - (min * 65535.0) / (max -
// 								      min);

// 	if (VFC_DG[i][j] > max2)
// 	  max2 = VFC_DG[i][j];
// 	if (VFC_DG[i][j] < min2)
// 	  min2 = VFC_DG[i][j];
//       }
//   cout << "min = " << min << " max = " << max << endl;
//   cout << "min2 = " << min2 << " max2 = " << max2 << endl;


//   // equalization, using 16 bits depth
//   // http://en.wikipedia.org/wiki/Histogram_equalization#Implementation
//   if (equalize)
//     {
//       std::vector < double >hist;
//       hist.resize (65536);
//       for (int i = 0; i < h; ++i)
// 	for (int j = 0; j < w; ++j)
// 	  {
// 	    int id = static_cast < int >(VFC_DG[i][j]);
// 	    if (id >= 0 && id < 65536)
// 	      hist[id] += 1.0 / (w * h);
// 	  }

//       double cum = 0;
//       std::vector < double >cdf;
//       cdf.resize (65536);
//       for (int i = 0; i < 65536; ++i)
// 	{
// 	  cum += hist[i];
// 	  cdf[i] = cum;
// 	}

//       for (int i = 0; i < h; ++i)
// 	for (int j = 0; j < w; ++j)
// 	  {
// 	    int id = static_cast < int >(VFC_DG[i][j]);
// 	    if (id >= 0 && id < 65536)
// 	      VFC_DG[i][j] = cdf[id] * 255;
// 	  }
//     }
//   else
//     {
//       for (int i = 0; i < h; ++i)
// 	for (int j = 0; j < w; ++j)
// 	  VFC_DG[i][j] /= 255;
//     }

//   // median filtering (first or after the equalization??)
//   if (filterVFC_DG)
//     {
//       for (int t = 0; t < 2; ++t)
// 	{
// 	  std::vector < double >pool;
// 	  pool.resize (9);
// 	  for (int i = 1; i < h - 1; ++i)
// 	    for (int j = 1; j < w - 1; ++j)
// 	      {
// 		int pos = 0;
// 		for (int si = -1; si < 2; ++si)
// 		  for (int sj = -1; sj < 2; ++sj)
// 		    {
// 		      pool[pos] = VFC_DG[i + si][j + sj];
// 		      ++pos;
// 		    }

// 		std::sort (pool.begin (), pool.end ());
// 		VFC_DG[i][j] = pool[4];
// 	      }
// 	}
//     }

//   dmatToCvMat (VFC_DG, img);
// }



void
getVFC (string files_basepath,
	cv::Mat src,
	VFCType vfcType,
	dmat & u_res,
	dmat & v_res, double gamma, double sigma1, double sigma2,
        bool verbose)
{
  basepath = files_basepath;
  // svf_fname = basepath + "saved_vector_fields.csv";


  // // removing saved vector field files
  // remove (svf_fname.c_str ());


  cv::Mat filtered, grad_x, grad_y, grad;
  double otsu_th;

//      /// Load an image
//      string fname = basepath + imgname;
//      //string fname = name;
//      std::cout << fname << std::endl;
//      cv::Mat src = cv::imread( fname );

  /// Convert it to gray
  //cv::cvtColor( src, src, CV_RGB2GRAY );

  generateImages ( /*imgname, */ src, filtered, grad_x, grad_y, grad, otsu_th,
                   sigma1, sigma2, verbose);

  int r = std::max (src.rows, src.cols) / 2 + 1;

  // building the kernel for the vfc
  dmat mx, my, f, u, v;
  buildVFK (mx, my, r, gamma);


  //save_vf("kernel1", "Kernel vector field", mx, my);


  // do-while(0) used for break;s
  do
    {

      if (vfcType == STANDARD)
	{
	  normalize01 (f, grad);
	  convolute (mx, my, f, u, v);

	  //save_vf("vfc1-no_norm", "VFC no normalized", u, v);
	  break;
	}

      if (vfcType == STANDARD_INVERTED)
	{
	  normalize01 (f, grad);
	  convolute (mx, my, f, u, v);
	  invertField (u, v);

	  //save_vf("vfc1-inverted", "VFC inverted", u, v);
	  break;
	}

      if (vfcType == STANDARD_NORMALIZED)
	{
	  normalize01 (f, grad);
	  convolute (mx, my, f, u, v);
	  normalizeLength (u, v, 1.0);	//length = 1

	  //save_vf("vfc1", "VFC normalized", u, v);
	  break;
	}

      // anisotropic vfc
      normalize01 (f, filtered);
      invert01 (f);

      // performing convolution of the kernel and the image
      convolute (mx, my, f, u, v);

      // scaled normalized map
      normalizeLength (u, v, 0.5);	//length = 1
      //double z = 0.5; // scaling factor

      //save_vf("kernel2", "Displacement vector map", au, av);

      dmat m1x, m1y;
      buildVFK_anisotropic (u, v, m1x, m1y, gamma);

      normalize01 (f, grad);

      //dmat u2, v2;
      convolute (m1x, m1y, f, u, v);

      if (vfcType == ANISOTROPIC)
	{
          if (verbose)
          {
            std::cout << "using Anisotropic VFC" << std::endl;
          }
	  //save_vf("vfc2", "Anisotropic VFC no normalized", u, v);
	  break;
	}

      if (vfcType == ANISOTROPIC_INVERTED)
	{
          if (verbose)
          {
            std::cout << "using inverted Anisotropic VFC" << std::endl;
          }
	  invertField (u, v);
	  //save_vf("vfc2", "Anisotropic VFC no normalized inverted", u, v);
	  break;
	}

      normalizeLength (u, v, 1);

      if (vfcType == ANISOTROPIC_NORMALIZED)
	{
          if (verbose)
          {
            std::cout << "using Anisotropic VFC normalized" << std::endl;
          }
	  //save_vf("vfc2", "Anisotropic VFC normalized", u, v);
	  break;
	}

    }
  while (0);

  // dmat VFC_DG;
  // getVFC_DistanceGradient (VFC_DG, u, v, src,	// the name is misleading. we also use it for vfc image
  //       		   0,	//neigh_size,
  //       		   1,	//equalize,
  //       		   0,	//vfcConst, unused
  //       		   false);	// median filtering filterVFC_DG );

  clone (u_res, u);
  clone (v_res, v);
}

