#ifndef VFC_H
#define VFC_H

#include <vector>
#include <string>
#include "opencv2/imgproc/imgproc.hpp"

typedef std::vector < std::vector < double > > dmat;

typedef enum
{
    STANDARD,
    STANDARD_INVERTED,
    STANDARD_NORMALIZED,
    ANISOTROPIC,
    ANISOTROPIC_INVERTED,
    ANISOTROPIC_NORMALIZED
} VFCType;

void
getVFC (std::string files_basepath,
	cv::Mat src,
	VFCType vfcType,
	dmat & u_res,
	dmat & v_res, double gamma, double sigma1, double sigma2,
        bool verbose);

//void normalize01(dmat &f, Imagen *img);
void
invert01 (dmat & f);
void
buildVFK (dmat & x, dmat & y, int r, double a);
void
convolute (dmat const &kernelX,
	   dmat const &kernelY, dmat const &img, dmat & rX, dmat & rY);
void
normalizeLength (dmat & u, dmat & v, double length);
void
buildVFK_anisotropic (dmat & u, dmat & v,	// displacement map
		      dmat & m1x, dmat & m1y,	// kernel
		      double a);

void
convolute_k (dmat const &kernelX,
	     dmat const &kernelY, dmat const &img, dmat & rX, dmat & rY);

void
convolute_i (dmat const &kernelX,
	     dmat const &kernelY, dmat const &img, dmat & rX, dmat & rY);

void
init_plans (int img_width, int img_height);

void
init (dmat & v, int w, int h);

void
save_vfc_component (std::string file, dmat & m);

void
read_vfc_component (std::string filename, dmat & m, int width, int height);

void
saveDmatAsImage (std::string name, dmat const &f, double factor);

#endif
