function [tout16, phase1, newmask] = ...
    segmentationAS(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp, ...
    SOURCE_FOLDER, MODEL_TO_COMPARE,ImageModality)

%% AFFINE SEGMENTATION (SEGMENTATION THROUGH AFFINE REGISTRATION USING
%% SCATTER SEARCH AS OPTIMIZATION METHOD): NON-DETERMINISTIC METHOD

%% IMPORTANT NOTES:
% CONVERT FILES (reference, orig_im, and mask) TO THE PROPER FORMAT (mhd) WITH ConvertImages.exe image_file.
% THESE IMAGES SHOULD BE IN A DIFFERENT FOLDER THAN THE COMMONLY USED,
% SINCE THEY ARE USED ONLY WITH ELASTIX.

ELASTIX_GROUND_TRUTH = 'elastixFolders/ground-truth';
ELASTIX_MASKS = 'elastixFolders/masks';
ELASTIX_ROI_PROCESSED = 'elastixFolders/ROI_processed';
source_image_nameGT = sprintf('%s/%s.mhd',ELASTIX_GROUND_TRUTH,image_name);
if strcmp(ImageModality,'ISH')
    source_image_nameMask = sprintf('%s/reference%d_mask.mhd',ELASTIX_MASKS,REFERENCES(best_ref));
end;
source_image_nameROIprocessed = sprintf('%s/%s.mhd',ELASTIX_ROI_PROCESSED,image_name);
phase1 = 0.0;
phase2 = 0.0;

%% Check if there is a ground truth for this image
if (exist(source_image_nameGT, 'file') ~= 2) % it is not in ELASTIX_GROUND_TRUTH
    switch MODEL_TO_COMPARE % Select the proper .mat, and convert it to .png
        case 'consensus'
            fname = strcat(SOURCE_FOLDER,'/',image_name,'.mat');
        case 'intersection'
            fname = strcat(SOURCE_FOLDER,'/',image_name,'_intersection.mat');
        case 'union'
            fname = strcat(SOURCE_FOLDER,'/',image_name,'_union.mat');
        case 'STAPLE'
            fname = strcat(SOURCE_FOLDER,'/',image_name,'_STAPLE.mat');
    end
    if (exist(fname, 'file') == 2) % Check in SOURCE_FOLDER
        load(fname);
        switch MODEL_TO_COMPARE % Select the proper .mat, and convert it to .png
            case 'consensus'
                fileGT = new_image;
                clear new_image
            case 'intersection'
                fileGT = intersection_image;
                clear intersection_image
            case 'union'
                fileGT = union_image;
                clear union_image
            case 'STAPLE'
                fileGT = new_image;
                clear new_image
        end
        imwrite(fileGT, regexprep(fname, '.mat', '.png','ignorecase'),'png');
        clear fileGT
        % [STATUS,MESSAGE,MESSAGEID] = MOVEFILE(SOURCE,DESTINATION)
        [STATUS,MESSAGE,MESSAGEID] = movefile(strcat(SOURCE_FOLDER,'\',image_name,'.png'),strcat(ELASTIX_GROUND_TRUTH,'\',image_name,'.png'));
        if STATUS ~= 1
            disp('Problem moving the Ground Truth to the ELASTIX_GROUND_TRUTH folder before conversion to MHD format')
        else
            % Convert .png to .mhd
            command = ['elastixFiles\ConvertToMetaimage.exe ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\',image_name,'.png')];
            system(command); %execute external command
        end;
    else
        disp('There is no Ground Truth associated to this image')
    end
end

if strcmp(ImageModality,'ISH')
    %% Check if there is a ground truth for this reference
    if (exist(strcat(ELASTIX_GROUND_TRUTH,'/reference',num2str(REFERENCES(best_ref)),'.mhd'), 'file') ~= 2) % it is not in ELASTIX_GROUND_TRUTH
        switch MODEL_TO_COMPARE % Select the proper .mat, and convert it to .png
            case 'consensus'
                fname = strcat(SOURCE_FOLDER,'/reference',num2str(REFERENCES(best_ref)),'.mat');
            case 'intersection'
                fname = strcat(SOURCE_FOLDER,'/reference',num2str(REFERENCES(best_ref)),'_intersection.mat');
            case 'union'
                fname = strcat(SOURCE_FOLDER,'/reference',num2str(REFERENCES(best_ref)),'_union.mat');
            case 'STAPLE'
                fname = strcat(SOURCE_FOLDER,'/reference',num2str(REFERENCES(best_ref)),'_STAPLE.mat');
        end
        if (exist(fname, 'file') == 2) % Check in SOURCE_FOLDER
            load(fname);
            switch MODEL_TO_COMPARE % Select the proper .mat, and convert it to .png
                case 'consensus'
                    fileGT = new_image;
                    clear new_image
                case 'intersection'
                    fileGT = intersection_image;
                    clear intersection_image
                case 'union'
                    fileGT = union_image;
                    clear union_image
                case 'STAPLE'
                    fileGT = new_image;
                    clear new_image
            end
            imwrite(fileGT, regexprep(fname, '.mat', '.png','ignorecase'),'png');
            clear fileGT
            % [STATUS,MESSAGE,MESSAGEID] = MOVEFILE(SOURCE,DESTINATION)
            [STATUS,MESSAGE,MESSAGEID] = movefile(strcat(SOURCE_FOLDER,'/reference',num2str(REFERENCES(best_ref)),'.png'),...
                strcat(ELASTIX_GROUND_TRUTH,'/reference',num2str(REFERENCES(best_ref)),'.png'));
            if STATUS ~= 1
                disp('Problem moving the Ground Truth to the ELASTIX_GROUND_TRUTH folder before conversion to MHD format')
            else
                % Convert .png to .mhd
                command = ['elastixFiles\ConvertToMetaimage.exe ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),...
                    '/reference',num2str(REFERENCES(best_ref)),'.png')];
                system(command); %execute external command
            end;
        else
            disp('There is no Ground Truth associated to this reference')
        end
    end
    
    %% Check if there is a mask for the reference (we assume that all references are in the ground-truth folder)
    if (exist(source_image_nameMask, 'file') ~= 2)
        % Create mask from .mhd
        command = ['elastixFiles\CreateMask.exe ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.mhd')];
        system(command); %execute external command
        % move it to the ELASTIX_MASKS folder
        movefile(strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'_mask.mhd'),...
            strcat(strrep(ELASTIX_MASKS, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'_mask.mhd'));
        movefile(strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'_mask.zraw'),...
            strcat(strrep(ELASTIX_MASKS, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'_mask.zraw'));
    end
end;

if strcmp(ImageModality,'ISH')
    %% Check if there is a ROI preprocessed
    if (exist(source_image_nameROIprocessed, 'file') ~= 2)
        hip_img = im2double(orig_im);
        roi_img = preprocess_roiLocal(hip_img, 0); % 0 -> the size is preserved
        imwrite(roi_img, strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',image_name,'.png'));
        clear roi_img hip_img
        % Convert .png to .mhd
        command = ['elastixFiles\ConvertToMetaimage.exe ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',image_name,'.png')];
        system(command); %execute external command
    end
    
    %% Check if there is the GT preprocessed
    if (exist(strcat(ELASTIX_ROI_PROCESSED,'/reference',num2str(REFERENCES(best_ref)),'.mhd'), 'file') ~= 2)
        refImage = imread(strcat(SOURCE_FOLDER,'/reference',num2str(REFERENCES(best_ref)),'_ROI.png'));
        hip_img = im2double(refImage);
        roi_img = preprocess_roiLocal(hip_img, 0); % 0 -> the size is preserved
        imwrite(roi_img, strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.png'));
        clear roi_img hip_img
        % Convert .png to .mhd
        command = ['elastixFiles\ConvertToMetaimage.exe ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.png')];
        system(command); %execute external command
    end
end

%% Method
if strcmp(ImageModality,'ISH')
    flag = (sum((exist(source_image_nameGT, 'file') + exist(source_image_nameMask, 'file') + exist(source_image_nameROIprocessed, 'file'))) == 6);
else
    flag = (sum((exist(source_image_nameGT, 'file') + exist(source_image_nameROIprocessed, 'file'))) == 4);
    databaseReferencesElastix_CT_MRI
    [truefalse, index] = ismember(image_name, ImagesTable);
    referenceToUseName = ReferencesTable(index);
end;
if flag
    % we have the files, and they're in the correct position
    
    out_dir_name1 = sprintf('%s/%s',out_dir_name, 'outputAffine');
    out_dir_name2 = sprintf('%s/%s',out_dir_name, 'outputTransformAffine');
    mkdir(out_dir_name2);
    
    tic
    if (exist(sprintf('%s/%s',out_dir_name, 'outputAffine'),'dir') ~= 7)
        % if we already have the outputAffine folder we can assume that AS
        % was aready executed
        mkdir(out_dir_name1);
        
        if strcmp(ImageModality,'ISH')
            % #First, perform the affine registration (with mask)
            %         elastix -m reference_image.mhd -f target_image.mhd -mMask
            %         mask_reference_image.mhd -out output_directory -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -mMask ', strcat(strrep(ELASTIX_MASKS, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'_mask.mhd'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineISH.txt'];
        elseif strcmp(ImageModality,'Lungs')
            %             if image_name(1) == 'l'
            % elastix -m reference_image -f target_image -out
            % output_directory_affine -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineLungs.txt'];
        elseif strcmp(ImageModality,'Knee')
            % elastix -m reference_image -f target_image -out
            % output_directory_affine -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineKnee.txt'];
            %             end;
        elseif strcmp(ImageModality,'MRI')
            % elastix -m reference_image -f target_image -out
            % output_directory_affine -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineMRI.txt'];
        end;
        system(command); %execute external command
    end;
    phase1 = toc;
    
    % # Apply the resulting transformation to the ground truth
    % transformix -in ground_truth_reference_image.mhd -out
    % other_output_directory -tp output_directory/TransformParameters.0.txt
    tic
    if strcmp(ImageModality,'ISH')
        command = ['elastixFiles\transformix.exe -in ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.mhd'), ...
            ' -out ', strrep(out_dir_name2, '/', '\'), ...
            ' -tp ', strcat(strrep(out_dir_name1, '/', '\'),'\TransformParameters.0.txt')];
    else
        command = ['elastixFiles\transformix.exe -in ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
            ' -out ', strrep(out_dir_name2, '/', '\'), ...
            ' -tp ', strcat(strrep(out_dir_name1, '/', '\'),'\TransformParameters.0.txt')];
    end;
    system(command); %execute external command
    phase2 = toc;
else
    sprintf('The image, the ground truth or the mask don''t exist for this target image: %s\n', image_name{1})
end;

tout16 = phase1 + phase2; % time: segmentation using affine registration (affine with scatter search)

%% SAVING IMAGES
% Convert MHD to PNG: from out_dir_name2 to out_dir_name
command = ['elastixFiles\ConvertToPNG.exe ', strcat(strrep(out_dir_name2, '/', '\'), '\result.mhd')];
system(command); %execute external command
atlasImageDeformed = imread(strcat(out_dir_name2, '\result.png'));
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_newmask_AS_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_newmask_AS_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI
end;
movefile(strcat(out_dir_name2, '\result.png'),hippo_filename);

atlasImageDeformed = double(atlasImageDeformed/255);

newmask = atlasImageDeformed;

% Save the .mat and .png of the result (the atlas deformed to fit the
% target image) to our_dir_name
save(strcat(out_dir_name,'/',image_name,'_AS.mat'),'newmask');
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_orig_im_newmask_AS_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_orig_im_newmask_AS_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI
end;

orig_im_resized = imresize(orig_im,size(newmask));
imwrite(im2double(orig_im_resized).*newmask, hippo_filename,'png');

