
function [tout18, newmask] = segmentationCV(orig_im, out_dir_name,image_name, ...
    SOURCE_FOLDER,REFERENCES,best_ref,timestamp, maskpoly, weightLengthTerm,sizeMedFilt,lambda1,lambda2,minimumSize,label)

%% SEGMENTATION USING CHAN & VESE MODEL: DETERMINISTIC METHOD

addpath('segmentationMethods\Chan-Vese')

tic

iterations = 500;
originalSize = size(orig_im);
I = im2uint8(orig_im);
imresizeSize = 500; % 1000
bigImage=0;
if (sum(originalSize > 2*imresizeSize)) > 0
    I = imresize(I, [imresizeSize imresizeSize]); %[1000 1000]
    bigImage =1;
end;

I = medfilt2(I, [sizeMedFilt sizeMedFilt],'symmetric'); % [20 20] [10 10] [5 5]

% I = addborder(I, 15, 175, 'inner');%20 10
if maskpoly == 0
    phi2 = chenvese2(I,'whole',iterations,weightLengthTerm,lambda1,lambda2,'chan');%'multiphase' 'chan'
else
    if bigImage
        phi = imresize(maskpoly,[imresizeSize imresizeSize]);%[1000 1000]
        se = strel('ball',20,0);
    else
        phi = maskpoly;
        se = strel('ball',10,0);
    end;
    phi(phi ~= 0) = 1;
    
    maskpoly2 = imdilate(phi,se);
%     specialImages1 = {'Cutl2_30','Gfap_95_2307','Mbp_130'};
%     if ismember(image_name,specialImages1)
%         phi2 = chenvese2(I,'whole',iterations,weightLengthTerm,lambda1,lambda2,'chan');%'multiphase' 'chan'
%     else
        phi2 = chenvese2(I,maskpoly2,iterations,weightLengthTerm,lambda1,lambda2,'chan');%'multiphase' 'chan'
%     end;
end;

% Sometimes, I don't know why, CV returns the background as object. To fix
% it we calculate the dice coefficient and we keep the best one.
% if strcmp(ImageModality,'ISH')
%     backgroundSize = sum(sum(phi2==0));
%     foregroundSize = sum(sum(phi2==1));
%     if backgroundSize < foregroundSize
%         phi2 = 1-phi2;
%     end;
% end;
if bigImage %it means that we have resized the image
    [TP FP FN Dice1 H J]=validateResults(imresize(phi2, originalSize),image_name,SOURCE_FOLDER,out_dir_name, 0,label,0,'consensus',0);
    [TP FP FN Dice2 H J]=validateResults(1-imresize(phi2, originalSize),image_name,SOURCE_FOLDER,out_dir_name, 0,label,0,'consensus',0);
else
    [TP FP FN Dice1 H J]=validateResults(phi2,image_name,SOURCE_FOLDER,out_dir_name, 0,label,0,'consensus',0);
    [TP FP FN Dice2 H J]=validateResults(1-phi2,image_name,SOURCE_FOLDER,out_dir_name, 0,label,0,'consensus',0);    
end;
if Dice1>Dice2
    % refining some details to improve the results
    phi = phi2;%addborder(phi2, 20, 0, 'inner');% 25 20 10
else
    phi = 1-phi2;
end;

newmask = phi;
I = imresize(newmask, originalSize,'nearest');

newmask= double(I);
if best_ref ~= -1
    hippo_filename = sprintf('%s/%s_Final_%s_Config%d_%s_%s_%s_%s_%s.png',out_dir_name,image_name,label, REFERENCES(best_ref),timestamp,num2str(sizeMedFilt), num2str(weightLengthTerm),num2str(lambda1), num2str(lambda2));
else
    hippo_filename = sprintf('%s/%s_Final_%s_Config%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, timestamp,num2str(sizeMedFilt), num2str(weightLengthTerm),num2str(lambda1), num2str(lambda2));
end;
imwrite(newmask, hippo_filename,'png');


close all

outputImage = zeros(size(newmask));
CC = bwconncomp(newmask,8); % A scalar connectivity specifier must be 1, 4, 6, 8, 18, or 26.
numPixels = cellfun(@numel,CC.PixelIdxList);% number of pixels of each connected component
[ordered,idx] = sort(numPixels,'descend');
ordered(ordered < minimumSize) = 0;
% con_component=2; %minimum number of con_components
for c=1:size(idx,2)%con_component
    if (ordered(c)) ~= 0
        outputImage(CC.PixelIdxList{idx(c)}) = 1;
    end;
end

% CC = bwconncomp(newmask,8); % A scalar connectivity specifier must be 1, 4, 6, 8, 18, or 26.
% numPixels = cellfun(@numel,CC.PixelIdxList);% number of pixels of each connected component
% [ordered,idx] = sort(numPixels,'descend');
% con_component=2; %minimum number of con_components
% for c=1:con_component
%     outputImage(CC.PixelIdxList{idx(c)}) = 1;
% end

newmask = outputImage;

tout18 = toc;

%%   SAVE IMAGES
save(strcat(out_dir_name,'/',image_name,'_',label,'.mat'),'newmask');
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_orig_im_newmask_%s_%d_%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, REFERENCES(best_ref),timestamp,num2str(sizeMedFilt), num2str(weightLengthTerm),num2str(lambda1), num2str(lambda2)); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_orig_im_newmask_%s_%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, timestamp,num2str(sizeMedFilt), num2str(weightLengthTerm),num2str(lambda1), num2str(lambda2)); %save extracted ROI
end;
imwrite(im2double(orig_im).*newmask, hippo_filename,'png');

if best_ref ~= -1
    hippo_filename = sprintf('%s/%s_newmask_%s_%d_%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, REFERENCES(best_ref),timestamp,num2str(sizeMedFilt), num2str(weightLengthTerm),num2str(lambda1), num2str(lambda2)); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_newmask_%s_%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, timestamp,num2str(sizeMedFilt), num2str(weightLengthTerm),num2str(lambda1), num2str(lambda2)); %save extracted ROI
end;
imwrite(newmask, hippo_filename,'png');

