
function [tout17, newmask] = segmentationGAC(orig_im, out_dir_name,image_name, ...
    REFERENCES,best_ref,timestamp, maskpoly, countour_weight, expansion_weight, sizeMedFiltGAC,minimumSize,label,ImageModality)

%% SEGMENTATION USING GEODESIC ACTIVE CONTOURS: DETERMINISTIC METHOD

addpath('segmentationMethods\GeodesicActiveContours')

tic

iterations = 500;
originalSize = size(orig_im);
I = im2uint8(orig_im);
imresizeSize = 500; % 1000
bigImage=0;
if (sum(originalSize > 2*imresizeSize)) > 0
    I = imresize(I, [imresizeSize imresizeSize]); %[1000 1000]
    bigImage =1;
end;

I = medfilt2(I, [sizeMedFiltGAC sizeMedFiltGAC],'symmetric'); %[10 10] [20 20]

g = ac_gradient_map(I);

% if bigImage
%     g2 = addborder(g, 20, 0.8, 'inner');
% else
g2=g;
% end;

% figure, imshow(I)
if maskpoly == 0
    phi = ac_SDF_2D('rectangle', size(I), 5) ;%'chess_box', size(I), 5, 5);%
else
    % This option is not usually used since if I want to obtain, for example,
    % results with GAC and DSGAC, I need two different files. If not, I
    % will have only one of them per image.
    if bigImage
        phi = imresize(maskpoly,[imresizeSize imresizeSize]);
    else
        phi = maskpoly;%[1000 1000]
    end;
    phi(phi ~= 0) = 1;
    if bigImage
        se = strel('ball',40,0); %20
        phi = imdilate(phi,se);
        %         se = strel('ball',30,0);
        maskpoly2 = phi;
        %         maskpoly2 = imdilate(phi,se);
    else
        %         se = strel('ball',10,0);
        %         phi = imdilate(phi,se);
        if strcmp(ImageModality,'Lungs')
            se = strel('ball',30,0);%40 in Lungs, 100 in Knee, 10 in the others
        elseif strcmp(ImageModality,'Knee')
            se = strel('ball',35,0);%100
        else
            se = strel('ball',10,0);
        end;
        maskpoly2 = imdilate(phi,se);
    end;
    BW = edge(phi);
    phi = -bwdist(BW);
    phi(maskpoly2 == 1) = - phi(maskpoly2 == 1);
end;

% countour_weight = 3; %4
% expansion_weight = -1;%0.5; %-0.5
delta_t = 5; %5 4
show_result = 1;%0
phi = ac_GAC_model(g2, phi, countour_weight, expansion_weight, ...
    delta_t, iterations, show_result);

newmask = phi;
I = imresize(newmask, originalSize,'nearest');
I = im2uint8(I);
I(I<255) = 0;
I(I==255) = 1;

newmask= double(I);
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_Final_%s_Config%d_%s_%s_%s_%s.png',out_dir_name,image_name, label, REFERENCES(best_ref),timestamp,num2str(countour_weight), num2str(expansion_weight), num2str(sizeMedFiltGAC));
else
    hippo_filename = sprintf('%s/%s_Final_%s_Config_%s_%s_%s_%s.png',out_dir_name,image_name, label, timestamp,num2str(countour_weight), num2str(expansion_weight), num2str(sizeMedFiltGAC));
end;
saveas(gcf,hippo_filename,'png')
close all

outputImage = zeros(size(newmask));

CC = bwconncomp(newmask,8); % A scalar connectivity specifier must be 1, 4, 6, 8, 18, or 26.
numPixels = cellfun(@numel,CC.PixelIdxList);% number of pixels of each connected component
[ordered,idx] = sort(numPixels,'descend');
ordered(ordered < minimumSize) = 0;
% con_component=2; %minimum number of con_components
for c=1:size(idx,2)%con_component
    if (ordered(c)) ~= 0
        outputImage(CC.PixelIdxList{idx(c)}) = 1;
    end;
end

% CC = bwconncomp(newmask,8); % A scalar connectivity specifier must be 1, 4, 6, 8, 18, or 26.
% numPixels = cellfun(@numel,CC.PixelIdxList);% number of pixels of each connected component
% [ordered,idx] = sort(numPixels,'descend');
% con_component=2; %minimum number of con_components
% for c=1:con_component
%     outputImage(CC.PixelIdxList{idx(c)}) = 1;
% end

newmask = outputImage;

tout17 = toc;

%%   SAVE IMAGES
save(strcat(out_dir_name,'/',image_name,'_',label,'.mat'),'newmask');
if best_ref ~= -1
    hippo_filename = sprintf('%s/%s_orig_im_newmask_%s_%d_%s_%s_%s_%s.png',out_dir_name,image_name, label, REFERENCES(best_ref),timestamp,num2str(countour_weight), num2str(expansion_weight), num2str(sizeMedFiltGAC)); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_orig_im_newmask_%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, timestamp,num2str(countour_weight), num2str(expansion_weight), num2str(sizeMedFiltGAC)); %save extracted ROI
end;
imwrite(im2double(orig_im).*newmask, hippo_filename,'png');

if best_ref ~= -1
    hippo_filename = sprintf('%s/%s_newmask_%s_%d_%s_%s_%s_%s.png',out_dir_name,image_name, label, REFERENCES(best_ref),timestamp,num2str(countour_weight), num2str(expansion_weight), num2str(sizeMedFiltGAC)); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_newmask_%s_%s_%s_%s_%s.png',out_dir_name,image_name, label, timestamp,num2str(countour_weight), num2str(expansion_weight), num2str(sizeMedFiltGAC)); %save extracted ROI
end;
imwrite(newmask, hippo_filename,'png');

