function [Wilcoxon_pValue] ...
    = nonParametricTwoMethods(M,significanceLevel,problemType,numberOfFunctions,...
    numberOfRunsPerFunction,methodUsed, outputPath, inputFile,numberOfMethodsToCompare, ...
    saveFigures, fileID,minimumAllowedNumberOfRuns)

% signrank(x) performs a two-sided signed rank test of the
% hypothesis that the data x come from a distribution whose median
% is zero
% signrank(x,y), for vector y the same length as x, performs a
% paired, two-sided test of the hypothesis that x - y comes from a
% distribution whose median is zero.
Wilcoxon_pValue = signrank(M(:,1),M(:,2),'alpha',significanceLevel);
label = sprintf('-- Results over all functions --\n');
disp(label); fprintf(fileID,label);
label = sprintf('p-value: %d\n',Wilcoxon_pValue);
disp(label); fprintf(fileID,label);

if Wilcoxon_pValue < significanceLevel
    medianMethods = [median(M(:,1)) median(M(:,2))];
    if problemType == 1 % maximization -> the method with highest median is the best
        [valuesSorted indexSorted] = sort(medianMethods,'descend');
        label = sprintf('%s vs %s -> There are significant differences: %s is the best method (maximizing)',char(methodUsed(1)), ...
            char(methodUsed(2)),char(methodUsed(indexSorted(1))));
        disp(label);
    else
        [valuesSorted indexSorted] = sort(medianMethods,'ascend');
        label = sprintf('%s vs %s -> There are significant differences: %s is the best method (minimizing)',...
            char(methodUsed(1)), char(methodUsed(2)),char(methodUsed(indexSorted(1))));
        disp(label);
    end;
else
    label = sprintf('%s vs %s -> There are NOT significant differences',char(methodUsed(1)), ...
        char(methodUsed(2)));
    disp(label);
end;
fprintf(fileID,strcat(label,'\n'));

%% IF WE HAVE MORE THAN ONE FUNCTION, WE CAN STUDY THE RESULTS ON
%% EVERY FUNCTION TO KNOW WHICH METHOD IS BETTER ALSO ON EACH ONE OF
%% THEM
if numberOfFunctions > 1 & numberOfRunsPerFunction >= minimumAllowedNumberOfRuns
    disp(' ')
    disp('Now, we study which method is better for every Function/Dataset: ')
    fprintf(fileID,'\n');
    % Now, we can analyze the effect of the function
    label = sprintf('-- Results per function --\n');
    disp(label); fprintf(fileID,label);
    
    for i=1:numberOfFunctions
        label = sprintf('- FUNCTION %d -',i);
        disp(label);
        fprintf(fileID,strcat(label,'\n'));
        
        % Since now we want to know the differences between methods for
        % each function, we have to test again the normality and the
        % homoscedasticity, because even if the whole column follows a
        % normal distribution, the different 'pieces' of it maybe not.
        temporalDataMatrix = [M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1),M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,2)];
        [KolmogorovSmirnoff_pValues normallyDistributed] = normalityCheck(temporalDataMatrix,significanceLevel,...
            numberOfMethodsToCompare,methodUsed,outputPath,strcat('Function',num2str(i),'_',inputFile), saveFigures, fileID);
        [Bartlett_pValue homoscedasticity] = homoscedasticityCheck(temporalDataMatrix,significanceLevel, fileID);
        
        if homoscedasticity & normallyDistributed
            [h Ttest_pValue] = ttest(M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1),...
                M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,2),significanceLevel);
            if Ttest_pValue < significanceLevel
                meanMethods = [mean(M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1)) mean(M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,2))];
                if problemType == 1 % maximization -> the method with highest median is the best
                    [valuesSorted indexSorted] = sort(meanMethods,'descend');
                    label = sprintf('%s vs %s -> There are significant differences in Function/Dataset %d: %s is the best method (maximizing using t-test)',char(methodUsed(1)), ...
                        char(methodUsed(2)),i,char(methodUsed(indexSorted(1))));
                    disp(label);
                else
                    [valuesSorted indexSorted] = sort(meanMethods,'ascend');
                    label = sprintf('%s vs %s -> There are significant differences in Function/Dataset %d: %s is the best method (minimizing using t-test)',char(methodUsed(1)), ...
                        char(methodUsed(2)),i,char(methodUsed(indexSorted(1))));
                    disp(label);
                end;
            else
                label = sprintf('There are NOT significant differences in Function/Dataset %d',i);
                disp(label);
            end;
        else
            Wilcoxon_pValue = signrank(M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1),M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,2),'alpha',significanceLevel);
            if Wilcoxon_pValue < significanceLevel
                medianMethods = [median(M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1)) median(M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,2))];
                if problemType == 1 % maximization -> the method with highest median is the best
                    [valuesSorted indexSorted] = sort(medianMethods,'descend');
                    label = sprintf('%s vs %s -> There are significant differences in Function/Dataset %d: %s is the best method (maximizing using wilcoxon)',char(methodUsed(1)), ...
                        char(methodUsed(2)),i,char(methodUsed(indexSorted(1))));
                    disp(label);
                else
                    [valuesSorted indexSorted] = sort(medianMethods,'ascend');
                    label = sprintf('%s vs %s -> There are significant differences in Function/Dataset %d: %s is the best method (minimizing using wilcoxon)',char(methodUsed(1)), ...
                        char(methodUsed(2)),i,char(methodUsed(indexSorted(1))));
                    disp(label);
                end;
            else
                label = sprintf('There are NOT significant differences in Function/Dataset %d',i);
                disp(label);
            end;
        end;
        fprintf(fileID,strcat(label,'\n'));
        fprintf(fileID,'\n');
        disp(' ');
    end; 
end;

fprintf(fileID,'\n');