
%% THIS FUNCTION RETURNS THE TP, FP AND FN RATES DOING THE VERIFICATION OF
%% SEGMENTATION RESULTS FOR THE SEGMENTATION THROUGH REGISTRATION METHODS.
%% IN THIS CASE, THE VALIDATION IS MADE BETWEEN THE TARGET DEFORMED, AND
%% THE GROUND TRUTH OF THE ATLAS.
function [TruePositivesPercentage FalsePositivesPercentage FalseNegativesPercentage DiceCoefficient Hausdorff JaccardIndex] = ...
    validateResultsElastix(newmask,image_name,sourceFolder,outFolder, label,MODEL_TO_COMPARE)

if isempty(findstr(image_name,'synthetic')) %synthetic only has consensus
    if strcmp(MODEL_TO_COMPARE,'consensus')
        load(strcat(sourceFolder,'/',image_name,'.mat'))
        intersection_image = new_image;
    elseif strcmp(MODEL_TO_COMPARE,'intersection')
        load(strcat(sourceFolder,'/',image_name,'_intersection.mat'))
    elseif strcmp(MODEL_TO_COMPARE,'union')
        load(strcat(sourceFolder,'/',image_name,'_union.mat'))
        intersection_image = union_image;
    elseif strcmp(MODEL_TO_COMPARE,'STAPLE')
        load(strcat(sourceFolder,'/',image_name,'_STAPLE.mat'))
        intersection_image = new_image;
    end;
else
    load(strcat(sourceFolder,'/',image_name,'.mat'))
    intersection_image = new_image;
end;
referenceSegmentation = intersection_image;
% automatically segmented image
automaticSegmentation = double(newmask);

% Control in the 'strange' case that the images are not binary
referenceSegmentation(referenceSegmentation >= 1) = 1;
referenceSegmentation(referenceSegmentation <=0) = 0;
referenceSegmentation(referenceSegmentation >= 0.5) = 1;
referenceSegmentation(referenceSegmentation < 0.5) = 0;
automaticSegmentation(automaticSegmentation >= 1) = 1;
automaticSegmentation(automaticSegmentation <=0) = 0;
automaticSegmentation(automaticSegmentation >= 0.5) = 1;
automaticSegmentation(automaticSegmentation < 0.5) = 0;

percentages = {};
referImages = {};
autoImages = {};


intersection = referenceSegmentation + automaticSegmentation;
intersection(intersection<2)=0;intersection(intersection>1)=1;

union = referenceSegmentation + automaticSegmentation;
union(union>0)=1; union(union <= 0)=0;

imageArea = size(intersection,1) * size(intersection,2);
AreaSegmentedAutomatically = (sum(sum(automaticSegmentation))/imageArea)*100;
AreaSegmentedManually = (sum(sum(referenceSegmentation))/imageArea)*100;
TruePositivesPercentage = (sum(sum(intersection))/sum(sum(referenceSegmentation)))*100;
TP = (sum(sum(intersection))/imageArea)*100; % TP with respect to the whole image. It can be used to calculate Dice Coefficient
percentage = TruePositivesPercentage;

NotSegmentedParts = referenceSegmentation-automaticSegmentation;
NotSegmentedParts(NotSegmentedParts <= 0) = 0;
NotSegmentedParts(NotSegmentedParts > 0) = 1;

FalseNegativesPercentage =  (sum(sum(NotSegmentedParts))/sum(sum(referenceSegmentation)))*100; % It should be the same as: 100 - TruePositivesPercentage
FN = (sum(sum(NotSegmentedParts))/imageArea)*100; % FN with respect to the whole image. It can be used to calculate Dice Coefficient

RGBNotSegmentedParts = repmat(uint8(255.*NotSegmentedParts),[1 1 3]);
RGBNotSegmentedParts(:,:,2) = 0;
RGBNotSegmentedParts(:,:,3) = 0;

SegmentingErrors = referenceSegmentation-automaticSegmentation;
SegmentingErrors(SegmentingErrors > 0) = 0;
SegmentingErrors(SegmentingErrors < 0) = 1;

FalsePositivesPercentage = (sum(sum(SegmentingErrors))/sum(sum(automaticSegmentation)))*100;
FP = (sum(sum(SegmentingErrors))/imageArea)*100; % FP with respect to the whole image. It can be used to calculate Dice Coefficient

% Dice Similarity Coefficient can be calculated in some of the
% following manners:
% (2*intersection)/(intersection + union)
% (2*TP)/(FP+TP+TP+FN) 
% (2*intersection)/(reference + automatic)
DiceCoefficient = ((2*sum(sum(intersection)))/(sum(sum(intersection))+sum(sum(union))));
% Jaccard Similarity Index
% Dice/(2-Dice)
% intersection/union
JaccardIndex = DiceCoefficient/(2-DiceCoefficient);

RGBSegmentingErrors = repmat(uint8(255.*SegmentingErrors),[1 1 3]);
RGBSegmentingErrors(:,:,1) = 0;
RGBSegmentingErrors(:,:,3) = 0;

RGBintersection = repmat(uint8(255.*intersection),[1 1 3]);
result = RGBintersection + RGBNotSegmentedParts + RGBSegmentingErrors;

result1 = edge(imresize(referenceSegmentation, [round(size(referenceSegmentation,1)/5) round(size(referenceSegmentation,2)/5)]), 'sobel');
result2 = edge(imresize(automaticSegmentation, [round(size(automaticSegmentation,1)/5) round(size(automaticSegmentation,2)/5)]), 'sobel');
[row1, col1, V1]=find(result1==1);
[row2, col2, V2]=find(result2==1);

% The  Hausdorff  distance represents a  measure of the
% spatial distance between two sets of points. It represents
% the farthest of the nearest point for each pixel in the
% boundary of the reference and the automatic segmentation.

minDists1 = zeros(1,size(row1,1));
for ii=1:size(row1,1)
    dists = zeros(1,size(row2,1));
    for j=1:size(row2,1)
        dist = sqrt((row1(ii)-row2(j))^2 + (col1(ii)-col2(j))^2);
        dists(j) = dist;
    end;
    minDists1(ii) = min(dists);
end;

minDists2 = zeros(1,size(row2,1));
for ii=1:size(row2,1)
    dists = zeros(1,size(row1,1));
    for j=1:size(row1,1)
        dist = sqrt((row2(ii)-row1(j))^2 + (col2(ii)-col1(j))^2);
        dists(j) = dist;
    end;
    minDists2(ii) = min(dists);
end;
% Here, we multiply the result per 5 since the images, to
% calculate the boundaries, were resized with a factor of 5.
Hausdorff = (max([minDists1,minDists2]))*5;

outImage = sprintf('%s/%s_%s_overlapped_Atlas(notInverted).png',outFolder,image_name,label);
imwrite(result, outImage,'png');

out_file_name = sprintf('%s/%s_%s_segmentationValues.txt',outFolder,image_name,label);
f = fopen(out_file_name,'w');
fprintf(f,'Hausdorff: %f\n\nDiceCoefficient: %f\n\nJaccardIndex: %f\n\nTruePositivesPercentage (AutomaticSegmentation and GroundTruth): %f\n\nFalsePositivesPercentage (percentage of the automatically segmented image that should not have been segmented): %f\n\nFalseNegativesPercentage (percentage of ground truth, that should have been segmented but it was not): %f\n', ...
    Hausdorff, DiceCoefficient,JaccardIndex, TruePositivesPercentage, FalsePositivesPercentage, FalseNegativesPercentage);
% catch TruePositivesPercentage
%     TruePositivesPercentage = 0.0;
%     FalseNegativesPercentage = 0.0;
%     FalsePositivesPercentage = 0.0;
%     out_file_name = sprintf('%s/%s.txt',outFolder,image_name);
%     f = fopen(out_file_name,'w');
%     fprintf(f,'Some error has happened. Maybe there is not ground truth associated to this image.\nTruePositivesPercentage: %f\n',TruePositivesPercentage);
%
% end;

clear referenceSegmentation automaticSegmentation NotSegmentedParts RGBNotSegmentedParts SegmentingErrors RGBSegmentingErrors RGBintersection

end





